
#
# This is the Common_settings.cmake file.
#

# ========== Setup existing build configurations (e.g. for Visual Studio) ==========
IF(NOT REPLACE_CMAKE_CONFIGURATION_TYPES)  # Set REPLACE_CMAKE_CONFIGURATION_TYPES if you want to add CMAKE_CONFIGURATION_TYPES manually using the cmake command line tool.
  IF(CMAKE_CONFIGURATION_TYPES)
     LIST(REMOVE_ITEM CMAKE_CONFIGURATION_TYPES MinSizeRel)
     LIST(REMOVE_ITEM CMAKE_CONFIGURATION_TYPES RelWithDebInfo)
     LIST(REMOVE_DUPLICATES CMAKE_CONFIGURATION_TYPES)
     SET(CMAKE_CONFIGURATION_TYPES "${CMAKE_CONFIGURATION_TYPES}" CACHE STRING "Reset the configurations to what we need" FORCE)
  ENDIF()
ENDIF()

# ========== Setup existing build type (e.g. for make) ==========
# If the user specifies -DCMAKE_BUILD_TYPE on the command line, take their definition
# and dump it in the cache along with proper documentation, otherwise set CMAKE_BUILD_TYPE
# to Debug prior to calling PROJECT()
IF(DEFINED CMAKE_BUILD_TYPE)
   SET(CMAKE_BUILD_TYPE ${CMAKE_BUILD_TYPE} CACHE STRING "Please choose build type. Possible values are: Debug Release (DebugFixpoint ReleaseFixpoint)")
ELSE()
   SET(CMAKE_BUILD_TYPE "Debug" CACHE STRING "Please choose build type. Possible values are: Debug Release (DebugFixpoint ReleaseFixpoint)")
ENDIF()

# Set Debug as default build-type if none is set
IF("${CMAKE_BUILD_TYPE}" STREQUAL "")
    MESSAGE("Setting Debug as default build-type")
    SET(CMAKE_BUILD_TYPE "Debug" CACHE STRING "" FORCE)
ENDIF()

# ========== Some cache variables ==========
SET(GLOBAL_VERSION "2.6.0" CACHE STRING "Global Version" FORCE)
SET(ADDITIONAL_PREPROCESSOR_DEFINES "" CACHE STRING "Appends given defines to the platform defines list. Separate defines by using a semicolon")
SET(SHOW_BUILD_INFORMATION 0 CACHE BOOL "Displays some information about the configuration")

SET(EGAC_BUILD_EGML 1 CACHE BOOL "eC only?")
SET(EGAC_BUILD_IMAGELIBS 0 CACHE BOOL "Include imagelibs in binary?")
SET(EGML_NEON_OPTIMIZATION 0 CACHE BOOL "Activate neon-blits in eGML?")
SET(EGML_BUILD_NEONLOOPS 0 CACHE BOOL "Include neonloops as source in binary?")
SET(EGML_BUILD_NEONLOOPS_BENCHMARK 0 CACHE BOOL "Include neonloops benchmark as source in binary?")

# ========== Some more cache variables which must be set here do be predefined ==========
SET(GUILIANI_ADD_BLU_PLATFORM 0 CACHE BOOL "Adds BLU platform to project")
SET(GUILIANI_ADD_EGML_PLATFORM 1 CACHE BOOL "Adds eGML platform to project")
SET(GUILIANI_ADD_DAVE_PLATFORM 0 CACHE BOOL "Adds D/AVE 2D platform to project")
SET(GUILIANI_ADD_DAVEHD_PLATFORM 0 CACHE BOOL "Adds D/AVE HD platform to project")
SET(GUILIANI_ADD_DIRECTFB_PLATFORM 0 CACHE BOOL "Adds directfb platform to project")
SET(GUILIANI_ADD_DRM_PLATFORM 0 CACHE BOOL "Adds drm platform to project")
SET(GUILIANI_ADD_VULKAN_PLATFORM 0 CACHE BOOL "Adds Vulkan platform to project")
SET(GUILIANI_ADD_DUMMY_FILESYSTEM_WRAPPER CACHE BOOL "Adds dummy filesystem wrapper to project (Required for platforms without native filesystem support)")
SET(GUILIANI_ADD_DUMMY_THREAD_WRAPPER CACHE BOOL "Adds dummy thread wrapper to project (Required for platforms without threading support)")
SET(GUILIANI_ADD_FREETYPE_PLATFORM 1 CACHE BOOL "Adds FreeType platform to project")
SET(GUILIANI_ADD_GLYPHLIB_PLATFORM 0 CACHE BOOL "Adds GlyphLib platform to project")
SET(GUILIANI_ADD_GL_PLATFORM 0 CACHE BOOL "Adds GL-platform to project")
SET(GUILIANI_ADD_OPENVG_PLATFORM 0 CACHE BOOL "Adds OpenVG platform to project")
SET(GUILIANI_ADD_WAYLAND_PLATFORM 0 CACHE BOOL "Adds WAyland platform to project")
SET(GUILIANI_USE_SDL2_PLATFORM 0 CACHE BOOL "use SDL2 instead of SDL")
SET(GUILIANI_USE_RTTI 1 CACHE BOOL "use RTTI")
SET(GUILIANI_USE_EXCEPTIONS 1 CACHE BOOL "use exception-handling")

# add packages by default
SET(GUILIANI_PACKAGE_ADD_ANIMATION 1 CACHE BOOL "Add Animation-package")
SET(GUILIANI_PACKAGE_ADD_CONNECTION 1 CACHE BOOL "Add Connection-package")
SET(GUILIANI_PACKAGE_ADD_STREAMING 1 CACHE BOOL "Add Streaming-package")
SET(GUILIANI_PACKAGE_ADD_MJPEG 1 CACHE BOOL "Add MJPEG-package")
SET(GUILIANI_PACKAGE_ADD_GESTURES 1 CACHE BOOL "Add Gesture-package")

# internal complexes
SET(LIBS_BUILD_BLU 0 CACHE BOOL "Build BLU from sources")
SET(LIBS_BUILD_GLYPHLIB 0 CACHE BOOL "Build GlyphLib from sources")
SET(LIBS_BUILD_DAVE 0 CACHE BOOL "Build D/AVE from sources")

# external libraries
SET(LIBS_BUILD_FREETYPE 0 CACHE BOOL "Build Freetype from sources (>= 2.6.5)")
SET(LIBS_BUILD_LIBJPEG 0 CACHE BOOL "Build libjpeg from sources (>= v8)")
SET(LIBS_BUILD_LIBPNG 0 CACHE BOOL "Build libpng from sources (>= 1.6.36)")
SET(LIBS_BUILD_ZLIB 0 CACHE BOOL "Build zlib from sources (>= 1.2.11)")

# unicode encoding
SET(EGAC_STRING_FORMAT "UTF16" CACHE STRING "use which unicode?")
set_property(CACHE EGAC_STRING_FORMAT PROPERTY STRINGS UTF16 UTF8)
