
#
# FreeRTOS-GNU-arm-rza3ul specific settings
#

# Board description
SET(CMAKE_BOARD_MANUFACTURER Renesas)
SET(CMAKE_BOARD_NAME RZA3UL)
SET(CMAKE_BOARD_OS FreeRTOS10)

ENABLE_LANGUAGE(ASM)

# Compiler flags (common)
SET(CMAKE_C_FLAGS "-mcpu=cortex-a55 -mcmodel=large -fmessage-length=0 -fsigned-char -ffunction-sections -fdata-sections -funsafe-math-optimizations -Wuninitialized -Wpointer-arith -Wshadow -Wlogical-op -Wno-attributes")

SET(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS} -Og -g -D_DEBUG")
SET(CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS} -Os -g0")

SET(CMAKE_CXX_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -fpermissive -fexceptions -frtti -fno-use-cxa-atexit")
SET(CMAKE_CXX_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -fpermissive -fexceptions -frtti -fno-use-cxa-atexit")

SET(CMAKE_ASM_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -x assembler-with-cpp -MMD -MP")
SET(CMAKE_ASM_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -x assembler-with-cpp -MMD -MP")

# Add subdirectory for BSP
IF(STREAMRUNTIME_SDK_PROJECT)
    SET(CMAKE_BSP_SUBDIRECTORY "${PROJECT_SOURCE_DIR}/../Renesas/BSP")
ELSE(STREAMRUNTIME_SDK_PROJECT)
    SET(CMAKE_BSP_SUBDIRECTORY "${PATH_BSP}/BSP")
ENDIF(STREAMRUNTIME_SDK_PROJECT)

# Linker flags
SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-M=${PROJECT_NAME}.map -nostartfiles -Xlinker --gc-sections")

# Linker Script
SET(LINKER_SCRIPT "-T ${CMAKE_BSP_SUBDIRECTORY}/script/fsp_qspi.ld -L${CMAKE_BSP_SUBDIRECTORY}/script")

# Include directories
TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza_cfg"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza_gen"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/src"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/aws"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/board"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/aws/FreeRTOS/FreeRTOS/Source"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/aws/FreeRTOS/FreeRTOS/Source/include"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/aws/FreeRTOS/FreeRTOS/Source/portable"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/aws/FreeRTOS/FreeRTOS/Source/portable/MemMang"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/board/rza3ul_smarc_qspi"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/inc/api"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/inc/instances"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/rm_freertos_port"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_gtm"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_ioport"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_lcdc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_mmu"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_riic_master"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_scif_uart"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_hhid"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/mcu"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS/Include"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS/Source"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS/SVD"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS/Include/iobitmasks"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/cmsis/Device/RENESAS/Include/iodefines"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/mcu/all"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/mcu/rza3ul"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic/src"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic/src/driver"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic/src/hw"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic/src/driver/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_basic/src/hw/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_hhid/src"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/r_usb_hhid/src/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza_cfg/aws"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza_cfg/fsp_cfg"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/rza_cfg/fsp_cfg/bsp"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/src/rza3ul_smarc"
)


# Link directories
TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${CMAKE_BINARY_DIR}/BSP"
)

# Preprocessor defines
SET(COMMON_DEFINES "eC_TARGET_COMPILER_GCC;eC_TARGET_ENV_FREERTOS;eC_TARGET_CPU_RZA3UL;eC_TARGET_CPU_ARM_NEON;PLATFORM_RENESAS_RZA3UL" CACHE INTERNAL "common defines" FORCE)

#------------------------------------------------------------------------------
ADD_DEFINITIONS(
-D_RZA_CORE=CA55_0
-D_RENESAS_RZA_
-DGUILIANI_TASK_STACK_SIZE=32768
)

# System calls
SET(SYSCALLS_C
  "${CMAKE_BSP_SUBDIRECTORY}/rza/fsp/src/bsp/mcu/all/bsp_sbrk.c"
  "${CMAKE_BSP_SUBDIRECTORY}/src/sections.c"
  "${CMAKE_BSP_SUBDIRECTORY}/src/syscalls.c"
  )

#------------------------------------------------------------------------------
# 4. define global stuff
TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${PATH_EGAC}/comps/eC/code/FreeRTOS"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/general"
)


IF(STREAMRUNTIME_SDK_PROJECT)
ELSE(STREAMRUNTIME_SDK_PROJECT)
  IF(EGML_NEON_OPTIMIZATION)
  #------------------------------------------------------------------------------
  # 4. define global stuff
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/comps/eGML/code/eGML_NEON"
    PRIVATE "${PATH_EGAC}/comps/eGML/code/eGML_NEON/easy_crc"
  )
  ENDIF(EGML_NEON_OPTIMIZATION)
ENDIF(STREAMRUNTIME_SDK_PROJECT)

#------------------------------------------------------------------------------
# 5. define executable file extension
set(CMAKE_EXECUTABLE_SUFFIX ".x")
