/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUI_TOGGLE_BEHAVIOUR__H_
#define GUI_TOGGLE_BEHAVIOUR__H_

#include "GUICmdBehaviour.h"

/** @brief Behaviour for repeating clicks on object
    Additional phases can be defined where the start-time of the phase is measured from the beginning of the previous phase
*/
class CGUIToggleBehaviour : public CGUICmdBehaviour
{
public:
    ///Default constructor
    CGUIToggleBehaviour();

    virtual ~CGUIToggleBehaviour();

    /** set initial state
    will also set the internal state
    @param bInitialState
    */
    void SetInitialState(const eC_Bool& bInitialState);

    /** get initial state
    @return initial state
    */
    eC_Bool GetInitialState() const;

    /** set the command which is executed on true
    @param pkCmd
    */
    void SetTrueCommand(CGUICommandPtr pkCmd);

    /** get the command which is executed on true
    @return command
    */
    CGUICommandPtr GetTrueCommand() const;

    /** set the command which is executed on false
    @param pkCmd
    */
    void SetFalseCommand(CGUICommandPtr pkCmd);

    /** get the command which is executed on false
    @return command
    */
    CGUICommandPtr GetFalseCommand() const;

#if defined(GUILIANI_STREAM_GUI)
    virtual void ReadFromStream();
#endif

#if defined(GUILIANI_WRITE_GUI)
    virtual void WriteToStream(const eC_Bool bWriteClassID = false);
#endif

protected:
    /**
        Executes the command for the given behaviour, if it is equal
        to the stored one.
        @param eFromBehaviour Behaviour slot from which this method was called.
    */
    virtual void ExecuteCommand(const BehaviourType_t& eFromBehaviour);

private:
    /** Copy-constructor.
        @param kSource Source object to be copied.
    */
    CGUIToggleBehaviour(const CGUIToggleBehaviour& kSource);

    /** Operator= method.
        @param kSource Source object to be copied.
        @return This instance.
    */
    CGUIToggleBehaviour& operator=(const CGUIToggleBehaviour& kSource);

private:
    /// initial state
    eC_Bool m_bInitialState;

    /// current state
    eC_Bool m_bCurrentState;

    /// Stores the command executed on true
    CGUICommandPtr m_pkCommandTrue;

    /// Stores the command executed on false
    CGUICommandPtr m_pkCommandFalse;
};

#endif
