/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUIEXCEPTION__H_
#define GUIEXCEPTION__H_

#include "eC_String.h"

/**
deactivating exception-support for Guiliani may save about 140kB of code-size on an ARM Cortex-M7

however it is not recommended to use code which was compiled with -fno-exceptions with a standard-library compiled with exception-support
see here: http://lopezruiz.net/2020/03/11-disabling-exception-handling-cpp.htm
and https://gitlab.kitware.com/cmake/cmake/-/issues/17991#note_424728

so it heavily depends on the used toolchain if execptions may be deactivated
*/

#ifdef GUILIANI_USE_EXCEPTIONS
#define GUI_TRY try
#define GUI_CATCH(x) catch(x)
#define GUI_CATCH_ALL catch(...)
#define GUI_THROW(x) throw x
#define GUI_RETHROW throw;
#else
#define GUI_TRY if (true)
#define GUI_CATCH(x) if (false)
#define GUI_CATCH_ALL if (false)
#define GUI_THROW(x)
#define GUI_RETHROW
#endif

/// This is the base class for Guiliani exceptions.

class CGUIException
{
public:
    /**
        Standard constructor.
    */
    CGUIException();

    /**
        Destructor.
    */
    virtual ~CGUIException();

    /**
        Report the message of the exception.
        @return The message of the exception.
    */
    virtual eC_String Report() const;
};

#endif
