/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUICOMPONENTMANAGER_H__
#define __GUICOMPONENTMANAGER_H__

#include "eC_Types.h"
#include "eC_TArrayStack.h"

#include "GUIConfigDebug.h"

// wrappers
class CGfxEnv;
class CGfxWrap;
class CFntWrap;
class CGUIInputMedia;
class CSndWrap;

// streaming
#ifdef GUILIANI_STREAM_GUI
class CGUIStreamReader;
#endif
#ifdef GUILIANI_WRITE_GUI
class CGUIStreamWriter;
#endif

// other
class CGUIAnimationHandler;
class CGUIClipboard;
class CGUICommandHandler;
#ifdef GUILIANI_CONNECTION_ACTIVE
class CGUIConnectionHandler;
#endif
class CGUIEventHandler;
class CGUIFileSysWrap;
class CGUIFactoryManager;
class CGUIGeneralResourceManager;
class CGUIImageLoader;
class CGUILocalisationHandler;
#ifdef GUILIANI_USE_PERFMON
class CGUIPerfMon;
#endif
class CGUIProfiler;
class CGUIProperties;
class CGUIResourceFileHandler;
class CGUIResourceManager;
class CGUITimer;
class CGUITrace;

/// central component-manager

/**
This is the central class which holds all relevant objects needed for running the GUI.
*/
class CGUIComponentManager
{
public:
    /** Get instance
    @return instance
    */
    static CGUIComponentManager& GetInstance();

    /** constructor
    */
    CGUIComponentManager();

    /** set instances from another manager
    @param pkComponentManager
    */
    void SetInstances(CGUIComponentManager* pkComponentManager);

    /** delete all instances
    */
    void DeleteInstances();

    /** set GfxEnv
    @param pkGfxEnv
    */
    void SetGfxEnv(CGfxEnv* pkGfxEnv);

    /** get GfxEnv
    @return instance to GfxEnv
    */
    CGfxEnv& GetGfxEnv();

    /** delete GfxEnv
    */
    void DeleteGfxEnv();

    /** check if GfxEnv has been set
    @return true if set
    */
    eC_Bool HasGfxEnv() const;

    /** set GfxWrap
    @param pkGfxWrap
    */
    void SetGfxWrap(CGfxWrap* pkGfxWrap);

    /** get GfxWrap
    @return instance to GfxWrap
    */
    CGfxWrap& GetGfxWrap();

    /** delete GfxWrap
    */
    void DeleteGfxWrap();

    /** check if GfxWrap is set
    @return true if set
    */
    eC_Bool HasGfxWrap() const;

    /** set FntWrap
    @param pkFntWrap
    */
    void SetFntWrap(CFntWrap* pkFntWrap);

    /** get FntWrap
    @return instance to FntWrap
    */
    CFntWrap& GetFntWrap();

    /** delete FntWrap
    */
    void DeleteFntWrap();

    /** check if FntWrap has been set
    @return true if set
    */
    eC_Bool HasFntWrap() const;

    /** set input-media
    @param pkInputMedia
    */
    void SetInputMedia(CGUIInputMedia* pkInputMedia);

    /** get input-media
    @return instance to input-media
    */
    CGUIInputMedia& GetInputMedia();

    /** delete input-media
    */
    void DeleteInputMedia();

    /** check if input-media has been set
    @return true if set
    */
    eC_Bool HasInputMedia() const;

    /** set SndWrap
    @param pkSndWrap
    */
    void SetSndWrap(CSndWrap* pkSndWrap);

    /** get SndWrap
    @return instance to SndWrap
    */
    CSndWrap& GetSndWrap();

    /** delete SndWrap
    */
    void DeleteSndWrap();

    /** check if SndWrap has been set
    @return true if set
    */
    eC_Bool HasSndWrap() const;

#ifdef GUILIANI_STREAM_GUI
    /** set streamreader stack
    @param pkStreamReaderStack
    */
    void SetStreamReaderStack(eC_TArrayStack<CGUIStreamReader*>* pkStreamReaderStack);

    /** get streamreader stack
    @return streamreader stack
    */
    eC_TArrayStack<CGUIStreamReader*>* GetStreamReaderStack();

    /** get streamreader
    @return streamreader
    */
    CGUIStreamReader& GetStreamReader();
#endif

#ifdef GUILIANI_WRITE_GUI
    /** set streamwriter stack
    @param pkStreamWriterStack
    */
    void SetStreamWriterStack(eC_TArrayStack<CGUIStreamWriter*>* pkStreamWriterStack);

    /** get streamwriter stack
    @return streamwriter stack
    */
    eC_TArrayStack<CGUIStreamWriter*>* GetStreamWriterStack();

    /** get streamwriter
    @return streamwriter
    */
    CGUIStreamWriter& GetStreamWriter();
#endif

    /** set animation-handler
    @param pkAnimationHandler
    */
    void SetAnimationHandler(CGUIAnimationHandler* pkAnimationHandler);

    /** get animation-handler
    @return instance to animation-handler
    */
    CGUIAnimationHandler& GetAnimationHandler();

    /** delete animation-handler
    */
    void DeleteAnimationHandler();

    /** check if animation-handler has been set
    @return true if set
    */
    eC_Bool HasAnimationHandler() const;

    /** set clipboard-instance
    @param pkClipboard
    */
    void SetClipboard(CGUIClipboard* pkClipboard);

    /** get clipboard
    @return instance to clipboard
    */
    CGUIClipboard& GetClipboard();

    /** delete clipboard
    */
    void DeleteClipboard();

    /** check if clipboard has been set
    @return true if set
    */
    eC_Bool HasClipboard() const;

    /** set command-handler
    @param pkCommandHandler
    */
    void SetCommandHandler(CGUICommandHandler* pkCommandHandler);

    /** get command-handler
    @return instance to command-handler
    */
    CGUICommandHandler& GetCommandHandler();

    /** delete command-handler
    */
    void DeleteCommandHandler();

    /** check if command-handler has been set
    @return true if set
    */
    eC_Bool HasCommandHandler() const;

#ifdef GUILIANI_CONNECTION_ACTIVE
    /** set connection-handler
    @param pkConnectionHandler
    */
    void SetConnectionHandler(CGUIConnectionHandler* pkConnectionHandler);

    /** get connection-handler
    @return instance to connection-handler
    */
    CGUIConnectionHandler& GetConnectionHandler();

    /** delete connection-handler
    */
    void DeleteConnectionHandler();

    /** check if connection-handler has been set
    @return true if set
    */
    eC_Bool HasConnectionHandler() const;
#endif

    /** set event-handler
    @param pkEventHandler
    */
    void SetEventHandler(CGUIEventHandler* pkEventHandler);

    /** get event-handler
    @return instance to event-handler
    */
    CGUIEventHandler& GetEventHandler();

    /** delete eventhandler
    */
    void DeleteEventHandler();

    /** check if event-handler has been set
    @return true if set
    */
    eC_Bool HasEventHandler() const;

    /** set factory-manager
    @param pkFactoryManager
    */
    void SetFactoryManager(CGUIFactoryManager* pkFactoryManager);

    /** get factory-manager
    @return instance to factory-manager
    */
    CGUIFactoryManager& GetFactoryManager();

    /** delete factory-manager
    */
    void DeleteFactoryManager();

    /** check if factor-manager has been set
    @return true if set
    */
    eC_Bool HasFactoryManager() const;

    /** set file-system
    @param pkFileSystem
    */
    void SetFileSystem(CGUIFileSysWrap* pkFileSystem);

    /** get file-system
    @return instance to file-system
    */
    CGUIFileSysWrap& GetFileSystem();

    /** delete file-system
    */
    void DeleteFileSystem();

    /** check if file-system has been set
    @return true if set
    */
    eC_Bool HasFileSystem() const;

    /** set general resource manager
    @param pkGeneralResourceManager
    */
    void SetGeneralResourceManager(CGUIGeneralResourceManager* pkGeneralResourceManager);

    /** get general resource manager
    @return instance to general resource manager
    */
    CGUIGeneralResourceManager& GetGeneralResourceManager();

    /** delete general-resource manager
    */
    void DeleteGeneralResourceManager();

    /** check if general resource manager has been set
    @return true if set
    */
    eC_Bool HasGeneralResourceManager() const;

    /** set image-loader
    @param pkImageLoader
    */
    void SetImageLoader(CGUIImageLoader* pkImageLoader);

    /** get image-loader
    @return instance to image-loader
    */
    CGUIImageLoader& GetImageLoader();

    /** delete image-loader
    */
    void DeleteImageLoader();

    /** check if image-loader has been set
    @return true if set
    */
    eC_Bool HasImageLoader() const;

    /** set localisation-handler
    @param pkLocalisationHandler
    */
    void SetLocalisationHandler(CGUILocalisationHandler* pkLocalisationHandler);

    /** get localisation-handler
    @return instance to localisation-handler
    */
    CGUILocalisationHandler& GetLocalisationHandler();

    /** delete localisation-handler
    */
    void DeleteLocalisationHandler();

    /** check if localisation-handler has been set
    @return true if set
    */
    eC_Bool HasLocalisationHandler() const;

#ifdef GUILIANI_USE_PERFMON
    /** set performance-monitor
    @param pkPerfMon
    */
    void SetPerfMon(CGUIPerfMon* pkPerfMon);

    /** get performance-monitor
    @return instance to performance-monitor
    */
    CGUIPerfMon& GetPerfMon();

    /** delete performance-monitor
    */
    void DeletePerfMon();

    /** check if performance-monitor has been set
    @return true if set
    */
    eC_Bool HasPerfMon() const;
#endif

    /** set profiler
    @param pkProfiler
    */
    void SetProfiler(CGUIProfiler* pkProfiler);

    /** get profiler
    @return instance to profiler
    */
    CGUIProfiler& GetProfiler();

    /** delete profiler
    */
    void DeleteProfiler();

    /** check if profiler has been set
    @return true if set
    */
    eC_Bool HasProfiler() const;

    /** set property-handler
    @param pkProperties
    */
    void SetProperties(CGUIProperties* pkProperties);

    /** get property-handler
    @return instance to property-handler
    */
    CGUIProperties& GetProperties();

    /** delete property-handler
    */
    void DeleteProperties();

    /** check if property-handler has been set
    @return true if set
    */
    eC_Bool HasProperties() const;

    /** set resource-file handler
    @param pkResourceFileHandler
    */
    void SetResourceFileHandler(CGUIResourceFileHandler* pkResourceFileHandler);

    /** get resource-file handler
    @return instance to resource-file handler
    */
    CGUIResourceFileHandler& GetResourceFileHandler();

    /** delete resource-file handler
    */
    void DeleteResourceFileHandler();

    /** check if resource-file handler has been set
    @return true if set
    */
    eC_Bool HasResourceFileHandler() const;

    /** set resource-manager
    @param pkResourceManager
    */
    void SetResourceManager(CGUIResourceManager* pkResourceManager);

    /** get resource-manager
    @return instance to resource-manager
    */
    CGUIResourceManager& GetResourceManager();

    /** delete resource-manager
    */
    void DeleteResourceManager();

    /** check if resource-manager has been set
    @return true if set
    */
    eC_Bool HasResourceManager() const;

    /** set timer
    @param pkTimer
    */
    void SetTimer(CGUITimer* pkTimer);

    /** get timer
    @return instance to timer
    */
    CGUITimer& GetTimer();

    /** delete Timer
    */
    void DeleteTimer();

    /** check if Timer has been set
    @return true if set
    */
    eC_Bool HasTimer() const;

    /** set trace-system
    @param pkTrace
    */
    void SetTrace(CGUITrace* pkTrace);

    /** get trace-system
    @return instance to trace-system
    */
    CGUITrace& GetTrace();

    /** delete Trace
    */
    void DeleteTrace();

    /** check if Trace has been set
    @return true if set
    */
    eC_Bool HasTrace() const;

private:
    // wrappers
    CGfxEnv* m_pkGfxEnv;
    CGfxWrap* m_pkGfxWrap;
    CFntWrap* m_pkFntWrap;
    CGUIInputMedia* m_pkInputMedia;
    CSndWrap* m_pkSndWrap;

    // streaming
#ifdef GUILIANI_STREAM_GUI
    eC_TArrayStack<CGUIStreamReader*>* m_pkStreamReaderStack;
#endif

#ifdef GUILIANI_WRITE_GUI
    eC_TArrayStack<CGUIStreamWriter*>* m_pkStreamWriterStack;
#endif

    CGUIAnimationHandler* m_pkAnimationHandler;
    CGUIClipboard* m_pkClipboard;
    CGUICommandHandler* m_pkCommandHandler;
#ifdef GUILIANI_CONNECTION_ACTIVE
    CGUIConnectionHandler* m_pkConnectionHandler;
#endif
    CGUIEventHandler* m_pkEventHandler;
    CGUIFactoryManager* m_pkFactoryManager;
    CGUIFileSysWrap* m_pkFileSystem;
    CGUIGeneralResourceManager* m_pkGeneralResourceManager;
    CGUIImageLoader* m_pkImageLoader;
    CGUILocalisationHandler* m_pkLocalisationHandler;
#ifdef GUILIANI_USE_PERFMON
    CGUIPerfMon* m_pkPerfMon;
#endif
    CGUIProfiler* m_pkProfiler;
    CGUIProperties* m_pkProperties;
    CGUIResourceFileHandler* m_pkResourceFileHandler;
    CGUIResourceManager* m_pkResourceManager;
    CGUITimer* m_pkTimer;
    CGUITrace* m_pkTrace;

    static CGUIComponentManager* ms_pkInstance;
};

#endif
