/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/
#ifndef GUIFAKEMOUSECURSOR_H
#define GUIFAKEMOUSECURSOR_H

#include "eC_Types.h"
#include "GUIImageResource.h"

/// Software mouse cursor
/** The CGUIFakeMouseCursor class emulates a mouse cursor by software.
    To use it, call CreateInstance. If the mouse cursor isn't needed anymore call DeleteInstance.

    The image IMG_STDCTRL_FAKE_MOUSE_CURSOR can be used to supply an image used as mouse cursor.
    Otherwise a built-in cursor will be used.
*/
class CGUIFakeMouseCursor
{
public:
    /// Standard constructor
    CGUIFakeMouseCursor();

    /// Standard destructor
    ~CGUIFakeMouseCursor();

    /** Sets the image which is used to draw the mouse-cursor
    @param eCursorImage image to be used
    */
    void SetCursorImage(const ImageResource_t& eCursorImage);

    /// update position and invalidate
    void Update();

    /** Draws the mouse cursor
    @return true
    */
    eC_Bool DoDraw();

private:
    void Invalidate();

private:
    /// image used for cursor
    ImageResource_t m_eCursorImage;

    /// X position of mouse pointer
    eC_UInt m_uiPosX;

    /// Y position of mouse pointer
    eC_UInt m_uiPosY;

    eC_UInt m_uiWidth;
    eC_UInt m_uiHeight;

    /// width of rendered cursor
    static const eC_UInt m_cSW_CURSOR_WIDTH;

    /// height of rendered cursor
    static const eC_UInt m_cSW_CURSOR_HEIGHT;
};
#endif
