/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUI_MEM_LEAK_FUNCTIONS_H__
#define __GUI_MEM_LEAK_FUNCTIONS_H__

#include "GUIConfigDebug.h"

/** memory-tracing function to initialize info
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of callin gfunction
*/
void Guiliani_info(const char* pFile, int iLine, const char* pFunction);

#ifdef GUILIANI_OVERLOAD_CPP_FUNCS
/** memory-tracing function used with new() operator
    @param pcFile File name
    @param iLine Line in file
    @param pcFunction Name of calling function
    @param uiSize Size
*/
void* operator new(size_t uiSize, const char* pcFile, int iLine, const char* pcFunction);

/** memory-tracing function used with new[] operator
    @param pcFile File name
    @param iLine Line in file
    @param pcFunction Name of calling function
    @param uiSize Size
*/
void* operator new[](size_t uiSize, const char* pcFile, int iLine, const char* pcFunction);

/** memory-tracing function used with delete() operator
    @param Memory Pointer to allocated memory
*/
void operator delete(void* Memory) throw();

/** memory-tracing function used with delete[] operator
    @param Memory Pointer to allocated memory
*/
void operator delete[](void* Memory) throw();
#endif

/** memory-tracing function used with operator new
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param uiSize Size
*/
void* Guiliani_new(size_t uiSize, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with operator new[]
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param uiSize Size
*/
void* Guiliani_newArray(size_t uiSize, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with operator delete
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param pMemory Pointer to allocated memory
*/
void Guiliani_delete(void* pMemory, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with operator delete[]
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param pMemory Pointer to allocated memory
*/
void Guiliani_deleteArray(void* pMemory, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with malloc()
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param uiSize Size
*/
void* Guiliani_malloc(size_t uiSize, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with free()
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param pMemory Pointer to allocated memory
*/
void Guiliani_free(void* pMemory, const char* pFile, int iLine, const char* pFunction);

/** memory-tracing function used with realloc()
    @param pFile File name
    @param iLine Line in file
    @param pFunction Name of calling function
    @param pMemory Pointer to allocated memory
    @param uiSize Size
*/
void* Guiliani_realloc(void* pMemory, size_t uiSize, const char* pFile, int iLine, const char* pFunction);

#endif
