/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUIVECTOR3_H__
#define __GUIVECTOR3_H__

#include "eC_Types.h"

/** Helper-class for a 3-dimensional vector
*/
class GUIVector3
{
public:
    /** Constructor
    @param xVal
    @param yVal
    @param zVal
    */
    GUIVector3(eC_Value xVal, eC_Value yVal, eC_Value zVal);

    /** Scalar product
    @param rkVec vector
    @return scalar-product
    */
    eC_Value operator* (const GUIVector3& rkVec) const;

    /** Translate this vector in positive direction
    @param rkVec vector
    @return translated vector
    */
    GUIVector3 operator+= (const GUIVector3& rkVec);

    /** Translate this vector in negative direction
    @param rkVec vector
    @return translated vector
    */
    GUIVector3 operator-= (const GUIVector3& rkVec);

    /** Scale this vector
    @param rkVal scaling-factor
    @return scaled vector
    */
    GUIVector3 operator*= (const eC_Value& rkVal);

    /** Rotate point around x-axis (angle given in rad)
    @param vAngle
    */
    void XRotate(eC_Value vAngle);

    /** Rotate point around y-axis (angle given in rad)
    @param vAngle
    */
    void YRotate(eC_Value vAngle);

public:
    eC_Value m_vX; ///< x
    eC_Value m_vY; ///< y
    eC_Value m_vZ; ///< z
};

#endif
