/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __CUSTOM_EXTENSION_FACTORY_H__
#define __CUSTOM_EXTENSION_FACTORY_H__

#include "GUIFactory.h"

#include "GUIControlResource.h"
#include "GUIBehaviourResource.h"
#include "GUICommandResource.h"
#include "GUILayouterResource.h"

// can be used to exclude non-supported systems
#if defined(eC_TARGET_CPU_X86) || defined(eC_TARGET_CPU_X64)
#define PLUGIN_ACTIVE
#endif

// special linkage needed on Windows
#if defined(eC_TARGET_ENV_WIN32)
#define LINKAGE  __declspec(dllexport)
#else
#define LINKAGE
#endif

#define GETEXTENSIONSFACTORY CustomExtensionFactory::GetInstance()

/** An example for a custom factory.

    This factory must be able to create all custom extensions that are
    registered in the editor.
  */
class CustomExtensionFactory : public CGUIFactory
{
public:
    virtual ~CustomExtensionFactory();

    static CustomExtensionFactory* GetInstance();

public:
    virtual CGUIObject* CreateControl(const ControlClassID_t& eControlID);

    virtual CGUILayouter* CreateLayouter(const LayouterClassID_t& eLayouterID);

    virtual CGUIBehaviourDecorator* CreateBehaviour(const BehaviourClassID_t& eBehaviourID);

    virtual CGUICommand* CreateCommand(const CommandClassID_t& eCommandID);

    virtual CGUIText* CreateText(const TextTypeID_t& eTextType);

#ifdef GUILIANI_ANIMATION_ACTIVE
    virtual CGUIAnimation* CreateAnimation(const AnimationType_t& eAnimation);
#endif

protected:
    // default constructor is private due to singleton
    CustomExtensionFactory();

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CustomExtensionFactory(const CustomExtensionFactory& kSource);

    /** Assignment operator.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CustomExtensionFactory& operator=(const CustomExtensionFactory& kSource);

private:
    static CustomExtensionFactory* ms_pkInstance; ///< instance
};
#endif
