/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNT_WRAP_GLYPHLIB_DAVE_H__
#define __FNT_WRAP_GLYPHLIB_DAVE_H__

#include "FntWrapGlyphLib.h"
#include "FontCacheGlyphLib.h"

#include "GUIImageData.h"

/// Glyphlib font wrapper specialization using DAVE-API.

/**
This is the cached GlyphLib implementation of the font wrapper for DAVE.
It implements the text rendering using DAVE.
For details see base class.

this version will use a glyph-cache for every font which is currently requested.
if the font is compressed it will be uncompressed before storing the glyph into the cache.
*/

class CFntWrapGlyphLibDAVE : public CFntWrapGlyphLib
{
public:
    /** Create a FntWrap-GlyphLib object. It should be created
    in implementation after creation of GfxWrap.
    @param uiGlyphCacheInitialSize initial size for glyph-cache
    @param bGlyphCacheAutoResize if true the cache will be resized if necessary
    @param uiGlyphCacheResizeBy amount of glyph to be resized
    @return True if it is successfully created, False otherwise.
    */
    static eC_Bool CreateInstance(
        const eC_UInt& uiGlyphCacheInitialSize = 32,
        const eC_Bool& bGlyphCacheAutoResize = true,
        const eC_UInt& uiGlyphCacheResizeBy = 10);

protected:
    /** Renders a glyph to the screen.
    This function takes the 8 bit alpha image provided by GlyphLib adds color and blits it to the screen.
    @param vX1 The x coordinate where the glyph image is rendered.
    @param vY1 The y coordinate where the glyph image is rendered.
    @param kGlyphImage The glyph image containing a 8 bit alpha image of the glyph.
    */
    virtual void RenderGlyphToScreen(const eC_Value& vX1, const eC_Value& vY1, CGUIImageData& kGlyphImage);

    virtual void SetFont(const FontResource_t &eID);

    virtual void LoadFont(const FontResource_t &eFontID);

    /** Unloading is not supported by GlyphLib
    @param eFontID FontID to unload
    */
    virtual void UnloadFont(const FontResource_t &eFontID);

    virtual void GetGlyphMetrics(const eC_Char cChar, GlyphMetrics_t& kMetrics);

    virtual eC_Int GetAscender() const;
    virtual eC_Int GetDescender() const;
    virtual eC_Int GetInternalLeading() const;

    virtual eC_Bool SetNOFFonts(const eC_UInt uiNOFFonts);

private:
    /** Create a FntWrap-GlyphLib object. It should be created
    in implementation after creation of GfxWrap.
    @param uiGlyphCacheInitialSize initial size for glyph-cache
    @param bGlyphCacheAutoResize if true the cache will be resized if necessary
    @param uiGlyphCacheResizeBy amount of glyph to be resized
    */
    CFntWrapGlyphLibDAVE(
        const eC_UInt& uiGlyphCacheInitialSize,
        const eC_Bool& bGlyphCacheAutoResize,
        const eC_UInt& uiGlyphCacheResizeBy);

    virtual ~CFntWrapGlyphLibDAVE();

private:
    /** Copy-constructor.
    @param kSource Source object to be copied.
    */
    CFntWrapGlyphLibDAVE(const CFntWrapGlyphLibDAVE& kSource);

    /** Operator= method.
    @param kSource Source object to be copied.
    @return This instance.
    */
    CFntWrapGlyphLibDAVE& operator=(const CFntWrapGlyphLibDAVE& kSource);

    void RenderTextInternal(
        const eC_String *lpString,
        const TextOperation_t &eOperation,
        const eC_Value& vX1,
        const eC_Value& vY1,
        eC_Value& vWidth,
        eC_Value& vHeight,
        eC_Value vWidthMax,
        eC_UInt &uiNumFittingChars);

private:
    eC_UInt m_uiGlyphCacheInitialSize;
    eC_Bool m_bGlyphCacheAutoResize;
    eC_UInt m_uiGlyphCacheResizeBy;

    eC_TArray<CFontCacheGlyphLib*> m_apFontCache;
};
#endif
