/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUIBITMAPPLANE_DAVE_H__
#define __GUIBITMAPPLANE_DAVE_H__

#include "GUIBitmapPlane.h"

#include "GfxWrapDave.h"
#include "GUIImageData.h"
#include "GUIVector3.h"

/** 
NOTE: DAVE does NOT support rotated BitmapPlanes!
@ingroup GUILIANI_BITMAPPLANES
@brief The BitmapPlane Class for DAVE
*/
class CGUIBitmapPlane_DAVE : public CGUIBitmapPlane
{
    ///< friend for access
    friend class CGfxWrapDAVE;

protected:
    /** Constructor
        @param uiNewBitmapID Bitmap ID
        @param rkParentObject Parent object
        @param vWidth Width
        @param vHeight Height
    */
    CGUIBitmapPlane_DAVE(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        eC_Value vWidth, eC_Value vHeight);
        
    virtual ~CGUIBitmapPlane_DAVE();

    virtual void DoDraw();

    /** Please note: In DAVE only ONE alpha value for the whole plane is supported. This implementation takes the value in "ubTopLeft"
    @param ubTopLeft
    @param ubBottomLeft
    @param ubBottomRight
    @param ubTopRight
    */
    virtual void SetAlphaValues(
        const eC_UByte ubTopLeft,
        const eC_UByte ubBottomLeft,
        const eC_UByte ubBottomRight,
        const eC_UByte ubTopRight)
    {
        m_ubAlpha = ubTopLeft;
    }

    virtual void Clear(const eC_UByte ubRed, const eC_UByte ubGreen, const eC_UByte ubBlue, const eC_UByte ubAlpha);

    /** Retrieves the internal CGUIImageData instance, in which the actual bitmap data of the DAVE BitmapPlane is stored
    @return image-data
    */
    const CGUIImageData& GetImageData() { return m_kImageData; }

    /** Sets the internal CGUIImageData instance
    @param kImagedata
    */
    void SetImageData(const CGUIImageData& kImagedata) { m_kImageData = kImagedata; }

    /** Return data of bitmap-plane as array
    @param pkData pointer to data
    */
    virtual void GetData(eC_TArray<eC_UByte>* pkData);

private:
    void Init(eC_Value vWidth, eC_Value vHeight);

    /// Internal helper method to simplify access to the active dave device handle
    d2_device* GetDaveDevice();

private:
    /// This holds the bitmap information (size and raw image data) of this BitmapPlane
    CGUIImageData m_kImageData;

    /// Alpha value 0-255
    eC_UByte m_ubAlpha;
};

#endif
