/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNTWRAP_FREETYPE_GL_FONTATLAS__H__
#define __FNTWRAP_FREETYPE_GL_FONTATLAS__H__

#include "FntWrapGLFontAtlas.h"

//FreeType includes
#include "ft2build.h"
#include FT_GLYPH_H

#include "eC_Types.h"
#include "eC_TArray.h"
#include "GUIFontResource.h"

/** This class is used by FntWrapFreeTypeGL. It stores the texture-atlas information for one Font.
*/
class CFntWrapFreeTypeGLFontAtlas : public CFntWrapGLFontAtlas
{
public:
    /** Constructor
    @param reFontID ID of selected font.
    @param uiNumberOfRows Initial number of rows for the texture atlas.
    @param rkFTLib Pointer of initialized FreeType library.
    */
    CFntWrapFreeTypeGLFontAtlas(
        const FontResource_t& reFontID,
        const eC_UInt& uiNumberOfRows,
        const FT_Library& rkFTLib);

    /** Destructor */
    virtual ~CFntWrapFreeTypeGLFontAtlas();

    /** Returns the Freetype face object of this font
    @return this font's Freetype face object.
    */
    virtual void* GetFace() { return (void*)m_pkFace; }

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @return Struct of just added character.
    */
    virtual FontAtlasRecord_t AddGlyph(const eC_UInt &uiUnicode);

private:
    /** Initializes this class by opening the font face and setting parameters.
    @param eFontID ID of the font to be created.
    */
    void Init(const FontResource_t& reFontID, FT_Library pkFTLib);

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapFreeTypeGLFontAtlas(const CFntWrapFreeTypeGLFontAtlas& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapFreeTypeGLFontAtlas& operator=(const CFntWrapFreeTypeGLFontAtlas& kSource);

private:
    FT_Face m_pkFace;
};
#endif
