/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#if !defined GUIBITMAPGL__H_
#define GUIBITMAPGL__H_

#include "GUIBitmapPlane.h"

class CGfxWrapGL;

/** @brief The BitmapPlane Class for GL with pseudo offscreen rendering.
    @ingroup GUILIANI_BITMAPPLANES
*/
class CGUIBitmapPlane_GL : public CGUIBitmapPlane
{
    friend class CGfxWrapGL; ///< @todo to access private functions

public:
    void DoDraw();

    virtual void SetWidth(eC_Value vWidth, eC_Bool bExpandCentered = false);

    virtual void SetHeight(eC_Value vHeight, eC_Bool bExpandCentered = false);

    virtual void SetAlphaValues(
        const eC_UByte ubTopLeft,
        const eC_UByte ubBottomLeft,
        const eC_UByte ubBottomRight,
        const eC_UByte ubTopRight);

    virtual void SetTopLeftV(eC_Value vV);
    virtual eC_Value GetTopLeftV();

    virtual void SetBottomLeftV(eC_Value vV);
    virtual eC_Value GetBottomLeftV();

    virtual void SetBottomRightV(eC_Value vV);
    virtual eC_Value GetBottomRightV();

    virtual void SetTopRightV(eC_Value vV);
    virtual eC_Value GetTopRightV();

    /** Accessor for the OpenGL Texture ID.
        @return ID of the OpenGL texture
     */
    const eC_UInt GetTextureID() const;

    virtual eC_Value Get2DPlaneDistance();

    virtual void Clear(const eC_UByte ubRed, const eC_UByte ubGreen, const eC_UByte ubBlue, const eC_UByte ubAlpha);

    virtual void GetData(eC_TArray<eC_UByte>* pkData);

protected:
    /** Protected constructor
        @param uiNewBitmapID Bitmap plane ID
        @param rkParentObject Parent object
        @param vWidth Width
        @param vHeight Height
    */
    CGUIBitmapPlane_GL(GUIBitmapPlaneID_t uiNewBitmapID, const CGUIObject& rkParentObject, eC_Value vWidth, eC_Value vHeight);
    virtual ~CGUIBitmapPlane_GL();

    /// Generates and binds a texture to this bitmap plane.
    void CreateTexture();

    /** Deletes the corresponding texture id */
    void DeleteTexture();

    /// Inits the Texture Object by actually allocating memory for it.
    virtual void CreateRenderSurface();

    /// Called by the framework to allow derived classes to free memory allocated by the associated render surface.
    virtual void DeleteRenderSurface() {}

    /// Calculates the vertices in the vertex array.
    void UpdateVertices();

    virtual eC_Bool DrawToBitmap();

    virtual eC_Bool DrawToScreen();

protected:
    eC_Bool m_bComplete; ///< is bitmap-plane complete

private:
    eC_UInt m_uiGLTextureID;          /// OpenGL TextureID;
    eC_TArray<eC_Value> m_avVertices; /// array of the 4 OpenGL Vertices
    eC_TArray<eC_UByte> m_aubColor;   /// array holding the color and alpha information for each vertex
    eC_Value m_vDepthCorrection;
    eC_UByte m_ubAlpha;
};
#endif
