/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUIIMAGEDATAGL__H_
#define GUIIMAGEDATAGL__H_

#include "GUIImageData.h"

/// OpenGL Version of CGUIImageData for textures.
class CGUIImageDataGL : public CGUIImageData
{
public:
    /** CGUIImageData constructor */
    CGUIImageDataGL();

    /** CGUIImageData destrcutor */
    ~CGUIImageDataGL();

    /** Set Texture-ID
    @param uiTextureID The texture ID
    */
    void SetTextureID(const eC_UInt uiTextureID);

    /** Set Texture-Width (is power of 2)
    @param uiTexWidth The texture width
    */
    void SetTextureWidth(const eC_UInt uiTexWidth);

    /** Set Texture-Width (is power of 2)
    @param uiTexHeight The texture height
    */
    void SetTextureHeight(const eC_UInt uiTexHeight);

    /** Return Texture-ID
    @return texture-id
    */
    const eC_UInt GetTextureID() const;

    /** Get Texture-width
    @return texture-width
    */
    eC_UInt GetTextureWidth() const;

    /** Get Texture-height
    @return texture-height
    */
    eC_UInt GetTextureHeight() const;

    /** Get GL-color-format
    @return GL-color-format
    */
    eC_UInt GetColorFormat() const;

private:
    /** Copy-constructor
    Dummy implementation to avoid unintended use of compiler-generated default
    */
    CGUIImageDataGL(const CGUIImageDataGL& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default
    */
    CGUIImageData& operator=(const CGUIImageDataGL& kSource);

    eC_UInt m_uiTextureID; // OpenTexture identifier
    eC_UInt m_uiTextureWidth; // texture size for calculating texture coordinates
    eC_UInt m_uiTextureHeight;
};

#endif
