/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GFXWRAPOGL__H_
#define GFXWRAPOGL__H_

#include "GfxWrapGL.h"

/// OpenGL (non-'OpenGL-ES') graphic wrapper.
class CGfxWrapOGL : public CGfxWrapGL
{
public:
    /** CreateInstance
    @param pkGfxEnv GfxEnv to use
    @return Indicate whether by this call, an instance is created.
    The instance is created, if true, not created, otherwise.
    */
    static eC_Bool CreateInstance(CGfxEnv* pkGfxEnv);

    /** Creates a Screen with the given parameters.
    */
    void CreateScreen();

    virtual eC_Bool Refresh(const RefreshCall_t eRefreshCallType);

    virtual void ReadTextureData(
        const eC_UInt& uiTexID,
        const eC_UInt& uiXPos, const eC_UInt& uiYPos,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight, void* pkData);

protected:
    /** Constructor
        Protected because of the singleton pattern.
        @param pkGfxEnv GfxEnv to use
    */
    CGfxWrapOGL(CGfxEnv* pkGfxEnv);

    /** Destructor
        Protected because of the singleton pattern.
    */
    virtual ~CGfxWrapOGL(void);

    /** Init
        Intended to be executed in
        user specific OS implementations of the OGLwrapper.
    */
    void Init(void);

    /** De-Init
    */
    void DeInit(void);

    /**
    Will create a bitmap-plane using FBO if supported,
    otherwise the standard bitmap-plane GL will be used
    @param uiNewBitmapID
    @param rkParentObject
    @param vWidth
    @param vHeight
    @return bitmap-plane
    */
    virtual CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value& vWidth, const eC_Value& vHeight);

private:
    /** Copy-constructor.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CGfxWrapOGL(const CGfxWrapOGL& kSource);

    /** Operator= method.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CGfxWrapOGL& operator=(const CGfxWrapOGL& kSource);
};
#endif
