/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GFXWRAP_OGLES_H__
#define __GFXWRAP_OGLES_H__

#include "GfxWrapGL.h"

#ifdef GUILIANI_GFXOGLDEBUG
/// Helper macro for convenient checks for OpenGL and EGL errors in debug builds.
#define CHECK_GL_AND_EGL_ERROR  CGfxWrapGL::CheckGLError(__FUNCTION__); CEGLEnvironment::CheckEGLError(__FUNCTION__);
#else
#define CHECK_GL_AND_EGL_ERROR
#endif


/// OGLES graphics wrapper base implementation.

/** This graphic wrapper implements the OpenGL ES base functions.
    All other OGLES wrapper are derived from this class.
    This class should be used if the target supports OpenGL ES 1.x and doesn't support
    PBuffers. A workaround will be used instead.
*/
class CGfxWrapOGLES : public CGfxWrapGL
{
public:
    virtual eC_Bool Refresh(const RefreshCall_t eRefreshCallType);

    /** Helper used to create the screen itself
    */
    virtual void CreateScreen();

protected:
    /** Constructor
        @param[in] pkGfxEnv environment
    */
    CGfxWrapOGLES(CGfxEnv* pkGfxEnv);

    /** Destructor */
    virtual ~CGfxWrapOGLES();

    /** Method to initialize the graphic library
        @return returns True if the initialisation succeded, False otherwise.
    */
    virtual eC_Bool Init(void);

    /** Method to give the graphic library the opportunity to clean up
        @return returns True if the deinitialisation succeded, False otherwise.
    */
    virtual eC_Bool DeInit(void);
};

#endif //__GFXWRAPOGLES__H__
