/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __OGLES_API_H__
#define __OGLES_API_H__

// OpenGL ES
#if defined(GUILIANI_USE_OGLES2)
    #define GUILIANI_USE_GL_SHADERS 1
    #include <GLES2/gl2.h>
    #include <GLES2/gl2ext.h>
#elif defined(GUILIANI_USE_OGLES11)
    // OpenGL ES 1.1
    //include path for new versions of EGL (includes changed)
    #if defined SYSTEM_NAME_DARWIN //the header path is different on Darwin systems (iOS/MacOS)
        #include <OpenGLES/ES1/gl.h>
    #else
        #include <GLES/gl.h>
        #include <GLES/glext.h>
    #endif
#endif

// Fixpoint environments (embedded targets) usually only support OpenGL ES.
// Float is usually supported on (non-ES) OpenGL desktop computers.
// Therefore, we do a simple mapping:
// Fixpoint -> OpenGL ES
// Float -> OpenGL

#if defined eC_FIXPOINT && defined GUILIANI_USE_OGLES2
#error Fixpoint is not supported in OGL and OGLES2 mode!
#endif

// maps the gl calls
#if defined eC_FIXPOINT

#define GL_EC_VALUE GL_FIXED
#define glTexEnvV glTexEnvx
#define glScaleV glScalex
#define glColor4V glColor4x
#define glOrthoV glOrthox
#define glFrustumV glFrustumx
#define glTranslateV glTranslatex
#define glClearColorV glClearColorx
#define glTexParameterV glTexParameterx
#define glTexEnvV glTexEnvx
#define glLoadMatrixV(m) glLoadMatrixx((GLfixed*)(m))
#define glMultMatrixV(m) glMultMatrixx((GLfixed*)(m))
#define glLineWidthV glLineWidthx
#define glClearDepthV glClearDepthx
// Use self implemented routine here. No more dependencies to glu library,
// when gluPerspectivex is not needed.
#define gluPerspectiveV GETGFX_GL.SetPerspectiveProjection
#define glRotateV glRotatex
#define glNormal3V glNormal3x
#define glMaterialV glMaterialx
#define glMaterialVv(a, b, c) glMaterialxv(a, b, (GLfixed*)(c))
#define glLightV glLightx
#define glLightVv(a, b, c) glLightxv(a, b, (GLfixed*)(c))
#define glLightModelVv(a, b) glLightModelxv(a, (GLfixed*)(b))
#define glGetV(a, b) glGetFixedv(a, (GLfixed*)(b))

#else

#if GUILIANI_USE_GL_SHADERS == 1

// when using shaders some gl-function are getting re-mapped for simplicity
#define glTranslateV GETGFX_GL.SetTranslation
#define glColor4V(r, g, b, a) glUniform4f(glGetUniformLocation(GETGFX_GL.GetShaderID(), "Color"), eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glOrthoV GETGFX_GL.SetOrthoProjection
#define glFrustumV GETGFX_GL.SetFrustumProjection
#define glRotateV(a, b, c, d) GETGFX_GL.SetRotation(a, b, c, d)
#define glScaleV GETGFX_GL.SetScaling

#undef GL_PROJECTION
#define GL_PROJECTION CGfxWrapGL::MT_PROJECTION

#undef GL_MODELVIEW
#define GL_MODELVIEW CGfxWrapGL::MT_MODELVIEW

// re-map GL-constants to point to locations of vertex-attributes
#undef GL_VERTEX_ARRAY
#define GL_VERTEX_ARRAY 0

#undef GL_TEXTURE_COORD_ARRAY
#define GL_TEXTURE_COORD_ARRAY 1

#undef GL_COLOR_ARRAY
#define GL_COLOR_ARRAY 2

#define glMatrixMode GETGFX_GL.SetMatrixMode
#define glLoadIdentity GETGFX_GL.LoadIdentity
#define glPushMatrix GETGFX_GL.PushMatrix
#define glPopMatrix GETGFX_GL.PopMatrix
#define glLoadMatrixV GETGFX_GL.SetMatrix
#define glEnableClientState glEnableVertexAttribArray
#define glDisableClientState glDisableVertexAttribArray
#define glVertexPointer(a, b, c, d) glVertexAttribPointer(GL_VERTEX_ARRAY, a, b, GL_FALSE, c, d)
#define glTexCoordPointer(a, b, c, d) glVertexAttribPointer(GL_TEXTURE_COORD_ARRAY, a, b, GL_FALSE, c, d)

#else

#define glTranslateV(x, y, z) glTranslatef(eC_ToFloat(x), eC_ToFloat(y), eC_ToFloat(z))
#define glColor4V(r, g, b, a) glColor4f(eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glRotateV(a, b, c, d) glRotatef(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d))
#define glTexEnvV glTexEnvf
#define glScaleV glScalef
#define glLoadMatrixV(a) glLoadMatrixf(reinterpret_cast<const GLfloat*>(a))
#define glMultMatrixV glMultMatrixf
#define glNormal3V(x, y, z) glNormal3f(eC_ToFloat(x), eC_ToFloat(y), eC_ToFloat(z))
#define glMaterialV(a, b, c) glMaterialf(a, b, eC_ToFloat(c))
#define glMaterialVv glMaterialfv
#define glLightV(a, b, c) glLightf(a, b, eC_ToFloat(c))
#define glLightVv glLightfv
#define glLightModelVv glLightModelfv
#define glOrthoV(a, b, c, d, e, f) glOrthof(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d), eC_ToFloat(e), eC_ToFloat(f))
#define glFrustumV(a, b, c, d, e, f) glFrustumf(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d), eC_ToFloat(e), eC_ToFloat(f))

#endif

#define GL_EC_VALUE GL_FLOAT
#define glClearColorV(r, g, b, a) glClearColor(eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glTexParameterV(target, pname, param) glTexParameterf(eC_ToFloat(target), eC_ToFloat(pname), eC_ToFloat(param))
#define glLineWidthV glLineWidth
#define glClearDepthV(a) glClearDepth(eC_ToFloat(a))
#define gluPerspectiveV GETGFX_GL.SetPerspectiveProjection
#define glGetV glGetFloatv

#endif

#endif
