/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __OGL_API_H__
#define __OGL_API_H__

// OpenGL

#if defined(WIN32)
    #include <windows.h> // needed for gl.h on windows
    #define GuilianiGLGetProcAddress wglGetProcAddress
#elif defined(__linux__)
    #include <SDL.h>
    #define GuilianiGLGetProcAddress SDL_GL_GetProcAddress
#endif

#if defined SYSTEM_NAME_DARWIN
    #include <OpenGL/gl.h>
    #include <OpenGL/glext.h>
#else
    #include <GL/gl.h>
    #include <GL/glext.h>
#endif

// if version 2.0 of OpenGL is defined, we can use shaders instead of fixed function-pipeline
#ifdef GL_VERSION_2_0
    #define GUILIANI_USE_GL_SHADERS 1
#endif

#if defined (GUILIANI_USE_OGL)
#if !defined SYSTEM_NAME_DARWIN
// API mappings
// Use extension for VBO and FBO
//defines for the VBO extensions
#define glDeleteBuffers             CGfxWrapGL::glDeleteBuffers_Guiliani
#define glBindBuffer                CGfxWrapGL::glBindBuffer_Guiliani
#define glGenBuffers                CGfxWrapGL::glGenBuffers_Guiliani
#define glBufferData                CGfxWrapGL::glBufferData_Guiliani

//defines for the FBO extensions
#define glGenFramebuffers           CGfxWrapGL::glGenFramebuffers_Guiliani
#define glBindFramebuffer           CGfxWrapGL::glBindFramebuffer_Guiliani
#define glFramebufferTexture2D      CGfxWrapGL::glFramebufferTexture2D_Guiliani
#define glCheckFramebufferStatus    CGfxWrapGL::glCheckFramebufferStatus_Guiliani
#define glDeleteFramebuffers        CGfxWrapGL::glDeleteFramebuffers_Guiliani

#if GUILIANI_USE_GL_SHADERS == 1
// defines for shaders
#define glBlendEquationSeparate     CGfxWrapGL::glBlendEquationSeparate_Guiliani
#define glDrawBuffers               CGfxWrapGL::glDrawBuffers_Guiliani
#define glStencilOpSeperate         CGfxWrapGL::glStencilOpSeparate_Guiliani
#define glStencilFuncSeperate       CGfxWrapGL::glStencilFuncSeparate_Guiliani
#define glStencilMaskSeperate       CGfxWrapGL::glStencilMaskSeparate_Guiliani
#define glAttachShader              CGfxWrapGL::glAttachShader_Guiliani
#define glBindAttribLocation        CGfxWrapGL::glBindAttribLocation_Guiliani
#define glCompileShader             CGfxWrapGL::glCompileShader_Guiliani
#define glCreateProgram             CGfxWrapGL::glCreateProgram_Guiliani
#define glCreateShader              CGfxWrapGL::glCreateShader_Guiliani
#define glDeleteProgram             CGfxWrapGL::glDeleteProgram_Guiliani
#define glDeleteShader              CGfxWrapGL::glDeleteShader_Guiliani
#define glDetachShader              CGfxWrapGL::glDetachShader_Guiliani
#define glDisableVertexAttribArray  CGfxWrapGL::glDisableVertexAttribArray_Guiliani
#define glEnableVertexAttribArray   CGfxWrapGL::glEnableVertexAttribArray_Guiliani
#define glGetActiveAttrib           CGfxWrapGL::glGetActiveAttrib_Guiliani
#define glGetActiveUniform          CGfxWrapGL::glGetActiveUniform_Guiliani
#define glGetAttachedShaders        CGfxWrapGL::glGetAttachedShaders_Guiliani
#define glGetAttribLocation         CGfxWrapGL::glGetAttribLocation_Guiliani
#define glGetProgramiv              CGfxWrapGL::glGetProgramiv_Guiliani
#define glGetProgramInfoLog         CGfxWrapGL::glGetProgramInfoLog_Guiliani
#define glGetShaderiv               CGfxWrapGL::glGetShaderiv_Guiliani
#define glGetShaderInfoLog          CGfxWrapGL::glGetShaderInfoLog_Guiliani
#define glGetShaderSource           CGfxWrapGL::glGetShaderSource_Guiliani
#define glGetUniformLocation        CGfxWrapGL::glGetUniformLocation_Guiliani
#define glGetUniformfv              CGfxWrapGL::glGetUniformfv_Guiliani
#define glGetUniformiv              CGfxWrapGL::glGetUniformiv_Guiliani
#define glGetVertexAttribdv         CGfxWrapGL::glGetVertexAttribdv_Guiliani
#define glGetVertexAttribfv         CGfxWrapGL::glGetVertexAttribfv_Guiliani
#define glGetVertexAttribiv         CGfxWrapGL::glGetVertexAttribiv_Guiliani
#define glGetVertexAttribPointerv   CGfxWrapGL::glGetVertexAttribPointerv_Guiliani
#define glIsProgram                 CGfxWrapGL::glIsProgram_Guiliani
#define glIsShader                  CGfxWrapGL::glIsShader_Guiliani
#define glLinkProgram               CGfxWrapGL::glLinkProgram_Guiliani
#define glShaderSource              CGfxWrapGL::glShaderSource_Guiliani
#define glUseProgram                CGfxWrapGL::glUseProgram_Guiliani
#define glUniform1f                 CGfxWrapGL::glUniform1f_Guiliani
#define glUniform2f                 CGfxWrapGL::glUniform2f_Guiliani
#define glUniform3f                 CGfxWrapGL::glUniform3f_Guiliani
#define glUniform4f                 CGfxWrapGL::glUniform4f_Guiliani
#define glUniform1i                 CGfxWrapGL::glUniform1i_Guiliani
#define glUniform2i                 CGfxWrapGL::glUniform2i_Guiliani
#define glUniform3i                 CGfxWrapGL::glUniform3i_Guiliani
#define glUniform4i                 CGfxWrapGL::glUniform4i_Guiliani
#define glUniform1fv                CGfxWrapGL::glUniform1fv_Guiliani
#define glUniform2fv                CGfxWrapGL::glUniform2fv_Guiliani
#define glUniform3fv                CGfxWrapGL::glUniform3fv_Guiliani
#define glUniform4fv                CGfxWrapGL::glUniform4fv_Guiliani
#define glUniform1iv                CGfxWrapGL::glUniform1iv_Guiliani
#define glUniform2iv                CGfxWrapGL::glUniform2iv_Guiliani
#define glUniform3iv                CGfxWrapGL::glUniform3iv_Guiliani
#define glUniform4iv                CGfxWrapGL::glUniform4iv_Guiliani
#define glUniformMatrix2fv          CGfxWrapGL::glUniformMatrix2fv_Guiliani
#define glUniformMatrix3fv          CGfxWrapGL::glUniformMatrix3fv_Guiliani
#define glUniformMatrix4fv          CGfxWrapGL::glUniformMatrix4fv_Guiliani
#define glValidateProgram           CGfxWrapGL::glValidateProgram_Guiliani
#define glVertexAttrib1d            CGfxWrapGL::glVertexAttrib1d_Guiliani
#define glVertexAttrib1dv           CGfxWrapGL::glVertexAttrib1dv_Guiliani
#define glVertexAttrib1f            CGfxWrapGL::glVertexAttrib1f_Guiliani
#define glVertexAttrib1fv           CGfxWrapGL::glVertexAttrib1fv_Guiliani
#define glVertexAttrib1s            CGfxWrapGL::glVertexAttrib1s_Guiliani
#define glVertexAttrib1sv           CGfxWrapGL::glVertexAttrib1sv_Guiliani
#define glVertexAttrib2d            CGfxWrapGL::glVertexAttrib2d_Guiliani
#define glVertexAttrib2dv           CGfxWrapGL::glVertexAttrib2dv_Guiliani
#define glVertexAttrib2f            CGfxWrapGL::glVertexAttrib2f_Guiliani
#define glVertexAttrib2fv           CGfxWrapGL::glVertexAttrib2fv_Guiliani
#define glVertexAttrib2s            CGfxWrapGL::glVertexAttrib2s_Guiliani
#define glVertexAttrib2sv           CGfxWrapGL::glVertexAttrib2sv_Guiliani
#define glVertexAttrib3d            CGfxWrapGL::glVertexAttrib3d_Guiliani
#define glVertexAttrib3dv           CGfxWrapGL::glVertexAttrib3dv_Guiliani
#define glVertexAttrib3f            CGfxWrapGL::glVertexAttrib3f_Guiliani
#define glVertexAttrib3fv           CGfxWrapGL::glVertexAttrib3fv_Guiliani
#define glVertexAttrib3s            CGfxWrapGL::glVertexAttrib3s_Guiliani
#define glVertexAttrib3sv           CGfxWrapGL::glVertexAttrib3sv_Guiliani
#define glVertexAttrib4Nbv          CGfxWrapGL::glVertexAttrib4Nbv_Guiliani;
#define glVertexAttrib4Niv          CGfxWrapGL::glVertexAttrib4Niv_Guiliani;
#define glVertexAttrib4Nsv          CGfxWrapGL::glVertexAttrib4Nsv_Guiliani;
#define glVertexAttrib4Nub          CGfxWrapGL::glVertexAttrib4Nub_Guiliani;
#define glVertexAttrib4Nubv         CGfxWrapGL::glVertexAttrib4Nubv_Guiliani;
#define glVertexAttrib4Nuiv         CGfxWrapGL::glVertexAttrib4Nuiv_Guiliani;
#define glVertexAttrib4Nusv         CGfxWrapGL::glVertexAttrib4Nusv_Guiliani;
#define glVertexAttrib4bv           CGfxWrapGL::glVertexAttrib4bv_Guiliani;
#define glVertexAttrib4d            CGfxWrapGL::glVertexAttrib4d_Guiliani
#define glVertexAttrib4dv           CGfxWrapGL::glVertexAttrib4dv_Guiliani
#define glVertexAttrib4f            CGfxWrapGL::glVertexAttrib4f_Guiliani
#define glVertexAttrib4fv           CGfxWrapGL::glVertexAttrib4fv_Guiliani
#define glVertexAttrib4iv           CGfxWrapGL::glVertexAttrib4iv_Guiliani
#define glVertexAttrib4s            CGfxWrapGL::glVertexAttrib4s_Guiliani
#define glVertexAttrib4sv           CGfxWrapGL::glVertexAttrib4sv_Guiliani
#define glVertexAttrib4ubv          CGfxWrapGL::glVertexAttrib4ubv_Guiliani;
#define glVertexAttrib4uiv          CGfxWrapGL::glVertexAttrib4uiv_Guiliani;
#define glVertexAttrib4usv          CGfxWrapGL::glVertexAttrib4usv_Guiliani
#define glVertexAttribPointer       CGfxWrapGL::glVertexAttribPointer_Guiliani
#endif

#endif
#endif

// Fixpoint environments (embedded targets) usually only support OpenGL ES.
// Float is usually supported on (non-ES) OpenGL desktop computers.
// Therefore, we do a simple mapping:
// Fixpoint -> OpenGL ES
// Float -> OpenGL

#if defined eC_FIXPOINT && defined GUILIANI_USE_OGL
#error Fixpoint is not supported in OGL and OGLES2 mode!
#endif

// maps the gl calls
#if defined eC_FIXPOINT

#define GL_EC_VALUE GL_FIXED
#define glTexEnvV glTexEnvx
#define glScaleV glScalex
#define glColor4V glColor4x
#define glOrthoV glOrthox
#define glFrustumV glFrustumx
#define glTranslateV glTranslatex
#define glClearColorV glClearColorx
#define glTexParameterV glTexParameterx
#define glTexEnvV glTexEnvx
#define glLoadMatrixV(m) glLoadMatrixx((GLfixed*)(m))
#define glMultMatrixV(m) glMultMatrixx((GLfixed*)(m))
#define glLineWidthV glLineWidthx
#define glClearDepthV glClearDepthx
// Use self implemented routine here. No more dependencies to glu library,
// when gluPerspectivex is not needed.
#define gluPerspectiveV GETGFX_GL.SetPerspectiveProjection
#define glRotateV glRotatex
#define glNormal3V glNormal3x
#define glMaterialV glMaterialx
#define glMaterialVv(a, b, c) glMaterialxv(a, b, (GLfixed*)(c))
#define glLightV glLightx
#define glLightVv(a, b, c) glLightxv(a, b, (GLfixed*)(c))
#define glLightModelVv(a, b) glLightModelxv(a, (GLfixed*)(b))
#define glGetV(a, b) glGetFixedv(a, (GLfixed*)(b))

#else

#if GUILIANI_USE_GL_SHADERS == 1

// when using shaders some gl-function are getting re-mapped for simplicity
#define glTranslateV GETGFX_GL.SetTranslation
#define glColor4V(r, g, b, a) glUniform4f(glGetUniformLocation(GETGFX_GL.GetShaderID(), "Color"), eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glOrthoV GETGFX_GL.SetOrthoProjection
#define glFrustumV GETGFX_GL.SetFrustumProjection
#define glRotateV(a, b, c, d) GETGFX_GL.SetRotation(a, b, c, d)
#define glScaleV GETGFX_GL.SetScaling

#undef GL_PROJECTION
#define GL_PROJECTION CGfxWrapGL::MT_PROJECTION

#undef GL_MODELVIEW
#define GL_MODELVIEW CGfxWrapGL::MT_MODELVIEW

// re-map GL-constants to point to locations of vertex-attributes
#undef GL_VERTEX_ARRAY
#define GL_VERTEX_ARRAY 0

#undef GL_TEXTURE_COORD_ARRAY
#define GL_TEXTURE_COORD_ARRAY 1

#define glMatrixMode GETGFX_GL.SetMatrixMode
#define glLoadIdentity GETGFX_GL.LoadIdentity
#define glPushMatrix GETGFX_GL.PushMatrix
#define glPopMatrix GETGFX_GL.PopMatrix
#define glLoadMatrixV GETGFX_GL.SetMatrix
#define glEnableClientState glEnableVertexAttribArray
#define glDisableClientState glDisableVertexAttribArray
#define glVertexPointer(a, b, c, d) glVertexAttribPointer(GL_VERTEX_ARRAY, a, b, GL_FALSE, c, d)
#define glTexCoordPointer(a, b, c, d) glVertexAttribPointer(GL_TEXTURE_COORD_ARRAY, a, b, GL_FALSE, c, d)

#else

#define glTranslateV(x, y, z) glTranslatef(eC_ToFloat(x), eC_ToFloat(y), eC_ToFloat(z))
#define glColor4V(r, g, b, a) glColor4f(eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glRotateV(a, b, c, d) glRotatef(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d))
#define glTexEnvV glTexEnvf
#define glScaleV glScalef
#define glLoadMatrixV(a) glLoadMatrixf(reinterpret_cast<const GLfloat*>(a))
#define glMultMatrixV glMultMatrixf
#define glNormal3V(x, y, z) glNormal3f(eC_ToFloat(x), eC_ToFloat(y), eC_ToFloat(z))
#define glMaterialV(a, b, c) glMaterialf(a, b, eC_ToFloat(c))
#define glMaterialVv glMaterialfv
#define glLightV(a, b, c) glLightf(a, b, eC_ToFloat(c))
#define glLightVv glLightfv
#define glLightModelVv glLightModelfv
#define glOrthoV(a, b, c, d, e, f) glOrtho(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d), eC_ToFloat(e), eC_ToFloat(f))
#define glFrustumV(a, b, c, d, e, f) glFrustum(eC_ToFloat(a), eC_ToFloat(b), eC_ToFloat(c), eC_ToFloat(d), eC_ToFloat(e), eC_ToFloat(f))

#endif

#define GL_EC_VALUE GL_FLOAT
#define glClearColorV(r, g, b, a) glClearColor(eC_ToFloat(r), eC_ToFloat(g), eC_ToFloat(b), eC_ToFloat(a))
#define glTexParameterV(target, pname, param) glTexParameterf(eC_ToFloat(target), eC_ToFloat(pname), eC_ToFloat(param))
#define glLineWidthV glLineWidth
#define glClearDepthV(a) glClearDepth(eC_ToFloat(a))
#define gluPerspectiveV GETGFX_GL.SetPerspectiveProjection
#define glGetV glGetFloatv

#endif

#endif
