/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FONTCACHE_GLYPHLIB_H__
#define __FONTCACHE_GLYPHLIB_H__

#include "FontCache.h"

// GlyphLib
#include "glyphlib.h"
#include "gui_font.h"

#include "eC_Types.h"
#include "eC_TArray.h"
#include "GUIImageData.h"
#include "GUIFontResource.h"

/** This class is used to cache glyphs loaded by GlyphLib. It stores the cache information for
one Font.
*/
class CFontCacheGlyphLib : public CFontCache
{
public:
    /** Constructor
    @param eFontID ID of selected font.
    @param pkFont Pointer of initialized FreeType library.
    @param uiInitialCacheSize initial number of simultaneously cached characters. If parameter
    equals zero the cache will automatically be resized to be
    large enough to contain all required characters.
    @param bAutoResize if true the cache gets resized if not big enough to hold new entries
    @param uiResizeBy number of elements which are added on resize
    */
    CFontCacheGlyphLib(
        gui_font_t* pkFont,
        const FontResource_t& eFontID,
        const eC_UInt& uiInitialCacheSize,
        const eC_Bool& bAutoResize,
        const eC_UInt& uiResizeBy);

    /** Destructor */
    virtual ~CFontCacheGlyphLib();

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @param uiPos Position in array where the character should be added.
    @return Struct of just added character.
    */
    virtual CachedChar_t AddChar(const eC_UInt &uiUnicode, const eC_UInt &uiPos);

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    */
    CFontCacheGlyphLib(const CFontCacheGlyphLib& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    @return
    */
    CFontCacheGlyphLib& operator=(const CFontCacheGlyphLib& kSource);

private:
    /** initialize
    */
    void Init();

private:
    /** handle of font
    */
    gui_font_t* m_pkFontHandle;
};

#endif
