/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics Corporation and/or its affiliates and may only
 * be used with products of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.
 * Renesas products are sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for
 * the selection and use of Renesas products and Renesas assumes no liability.  No license, express or implied, to any
 * intellectual property right is granted by Renesas.  This software is protected under all applicable laws, including
 * copyright laws. Renesas reserves the right to change or discontinue this software and/or this documentation.
 * THE SOFTWARE AND DOCUMENTATION IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND
 * TO THE FULLEST EXTENT PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY,
 * INCLUDING WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE
 * SOFTWARE OR DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR
 * DOCUMENTATION (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER,
 * INCLUDING, WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY
 * LOST PROFITS, OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/

#include "rza3ul_smarc_i2c.h"
#include "FreeRTOS.h"
#include "task.h"

void i2c_master_callback(i2c_master_callback_args_t *p_args);
static volatile uint32_t i2c_wait_flag = 0;

/**
 * @name       i2c_master_callback
 * @brief      Callback for I2C channel module
 * @param[in]  *p_args : I2C master callback data
 * @return     None
 */
void i2c_master_callback(i2c_master_callback_args_t *p_args)
{
    if( p_args->event == I2C_MASTER_EVENT_TX_COMPLETE )
    {
        i2c_wait_flag = 1;
    }
}

/**
 * @name       bsp_i2c_open
 * @brief      Configure I2C pins
 * @param[in]  none
 * @return     none
 */
void bsp_i2c_open(void)
{
    /* opening IIC master module */
    R_RIIC_MASTER_Open (g_i2c_master1.p_ctrl, g_i2c_master1.p_cfg);
}


/**
 * @name       bsp_i2c_write
 * @brief      Perform an i2c write access
 * @param[in]  *data : Data to be written
 * @param[in]  data_length : Data length
 * @return     None
 */
void bsp_i2c_write(uint8_t * data, uint32_t len)
{
    i2c_wait_flag = 0;

    /* Write data to EEPROM  */
    R_RIIC_MASTER_Write (g_i2c_master1.p_ctrl, data, len, false);

    while (i2c_wait_flag == 0)
    {
        vTaskDelay(1);
    }
}
