/*
 * mouse_api.c
 *
 *  Created on: 2022/05/17
 *      Author: a5116907
 */


#include "hal_data.h"
#include "arrow_data.h"
#include "rza3ul_smarc_mouse.h"

#define     TPEVT_MOVE_MINTHD_USB     (0UL)
#define     USB_POS_TYPE_ABSOLUTE     (0)
#define     USB_POS_TYPE_RELATIVE     (1)

#define     TPEVT_ENTRY_MAX         (16)

#define     TP_DEVADDR_SHIFT        (16)
#define     TP_DEVADDR_MASK         (0xFFFF0000)

#define     TP_PENSTATUS_RELEASE    (0)
#define     TP_PENSTATUS_PUSH       (1)

#define		TP_USE_UNKNOWN			(0)
#define		TP_USE_MOUSE			(1)
#define		TP_USE_TOUCH			(2)

/******************************************************************************
Structures
******************************************************************************/

typedef struct {
    uint32_t     buffer_size;         /* copy data size to TP task from USB */
} r_usb_keyboard_info_t;

typedef struct {
    uint32_t     buffer_size;         /* copy data size to TP task from USB */
    uint32_t     pos_data_type;       /* position data type (absolute / relative) */
    uint32_t     data_len_pos;        /* byte length of X coodinate value */ 
    uint32_t     data_offset_posX1;  /* position of X coordinate in buffer */
    uint32_t     data_offset_posY1;  /* position of Y coordinate in buffer */
    int32_t      dev_resolutionX;     /* X resolution value of device */
    int32_t      dev_max_posX;        /* max X coordinate value of device */
    int32_t      dev_resolutionY;     /* Y resolution value of device */
    int32_t      dev_max_posY;        /* max Y coordinate value of device */
    uint32_t     finger_info_pos;     /* position of finger info in buffer, touch only */
} r_usb_touch_info_t;

/******************************************************************************
Variable Externs
******************************************************************************/

/*! Touch Panel screen width */
static int32_t    ScreenWidth ;
/*! Touch Panel screen height */
static int32_t    ScreenHeight ;

static int32_t    InputMode = TP_USE_UNKNOWN;

typedef struct {
    uint32_t     buffer_size;         /* copy data size to TP task from USB */
    uint32_t     pos_data_type;       /* position data type (absolute / relative) */
    uint32_t     data_len_pos;        /* byte length of X coodinate value */ 
    uint32_t     data_offset_posX1;  /* position of X coordinate in buffer */
    uint32_t     data_offset_posY1;  /* position of Y coordinate in buffer */
    uint32_t     dev_resolutionX;     /* X resolution value of device */
    uint32_t     dev_max_posX;        /* max X coordinate value of device */
    uint32_t     dev_resolutionY;     /* Y resolution value of device */
    uint32_t     dev_max_posY;        /* max Y coordinate value of device */
    uint32_t     data_offset_click;  /* position of click data, mouse only */
    uint8_t      left_click_bit;      /* position of left click bit, mouse only */
    uint8_t      rigith_click_bit;    /* position of right click bit, mouse only */
    uint8_t      center_button_bit;   /* position of center button bit, mouse only */
    uint32_t     data_offset_wheel;  /* position of wheel in buffwheelX value, mouse only */ 
    uint32_t     cursor_speed;        /* mouse cursor speed magnification, mouse only */
} r_usb_mouse_info_t;

r_usb_touch_info_t usb_touch_info;
r_usb_mouse_info_t usb_mouse_info;

void TP_SetPositionUsbMouse( TP_TouchFinger_st *psTouch, TPEVT_COORDINATES *psLastCoords, int32_t *pnPenStatus, const uint8_t* const puReadBuf);
void TP_GetScreenSize( int32_t *pnWidth, int32_t *pnHeight );
void TP_SetScreenSize( int32_t pnWidth, int32_t pnHeight );
static void  TP_GetPositionData(const uint8_t* const puReadBuf, int32_t *PosX, int32_t *PosY, const uint32_t data_len, uint32_t posx_offset, uint32_t posy_offset );
static void TP_LineUpPos( const int16_t nXin, const int16_t nYin, int32_t *pnXout, int32_t *pnYout );
static int32_t TP_EventHandleUsbMouse( TpEvt_EntryType *eEvent, const int32_t* const nPosX, const int32_t* const nPosY, TPEVT_COORDINATES *psLastCoords );
static int32_t TP_CheckDiffMin( const TPEVT_COORDINATES* const psPox_1, const TPEVT_COORDINATES* const psPox_2, const uint32_t unThreshold );

void pointer_init()
{
    uint32_t * pbuff_32;
    uint32_t * src_pbuff_32;
    uint32_t width;
    uint32_t height;
    //pbuff_32 = (uint32_t *)fb_foreground[0];
    pbuff_32 = (uint32_t *)g_display0_cfg.input[1].p_base;
    src_pbuff_32 = (uint32_t *)&g_arrow_data[0];
    for( height = 0 ; height < DISPLAY_VSIZE_INPUT1 ; height++ )
    {
        for( width = 0 ; width < DISPLAY_HSIZE_INPUT1 ; width++ )
        {
            pbuff_32[ DISPLAY_HSIZE_INPUT1 * height + width] = src_pbuff_32[DISPLAY_HSIZE_INPUT1 * height + width]; // ARGB8888
        }
    }
}

void mouse_init(int32_t display_width, int32_t display_height)
{
    usb_mouse_info.buffer_size = 4;         /* copy data size to TP task from USB */
    usb_mouse_info.pos_data_type = USB_POS_TYPE_RELATIVE;       /* position data type (absolute / relative) */
    usb_mouse_info.data_len_pos = 1;        /* byte length of X coodinate value */ 
    usb_mouse_info.data_offset_posX1 = 1;  /* position of X coordinate in buffer */
    usb_mouse_info.data_offset_posY1 = 2;  /* position of Y coordinate in buffer */
    usb_mouse_info.dev_resolutionX = 0;     /* X resolution value of device */
    usb_mouse_info.dev_max_posX = (uint32_t)display_width;        /* max X coordinate value of device */
    usb_mouse_info.dev_resolutionY = 0;     /* Y resolution value of device */
    usb_mouse_info.dev_max_posY = (uint32_t)display_height;        /* max Y coordinate value of device */
    usb_mouse_info.data_offset_click = 0;  /* position of click data, mouse only */
    usb_mouse_info.left_click_bit = 0x01;      /* position of left click bit, mouse only */
    usb_mouse_info.rigith_click_bit = 0x02;    /* position of right click bit, mouse only */
    usb_mouse_info.center_button_bit = 0x04;   /* position of center button bit, mouse only */
    usb_mouse_info.data_offset_wheel = 3;  /* position of wheel in buffwheelX value, mouse only */ 
    usb_mouse_info.cursor_speed = 2;        /* mouse cursor speed magnification, mouse only */

    InputMode = TP_USE_MOUSE;
    TP_SetScreenSize(display_width, display_height);
}

void touch_init(int32_t display_width, int32_t display_height, int32_t touch_width, int32_t touch_height)
{
    usb_touch_info.buffer_size = 4;         /* copy data size to TP task from USB */
    usb_touch_info.pos_data_type = USB_POS_TYPE_ABSOLUTE;       /* position data type (absolute / relative) */
    usb_touch_info.data_len_pos = 2;        /* byte length of X coodinate value */
    usb_touch_info.data_offset_posX1 = 2;  /* position of X coordinate in buffer */
    usb_touch_info.data_offset_posY1 = 4;  /* position of Y coordinate in buffer */
    usb_touch_info.dev_resolutionX = touch_width;     /* X resolution value of device */
    usb_touch_info.dev_max_posX = display_width;        /* max X coordinate value of device */
    usb_touch_info.dev_resolutionY = touch_height;     /* Y resolution value of device */
    usb_touch_info.dev_max_posY = display_height;        /* max Y coordinate value of device */

    InputMode = TP_USE_TOUCH;
    TP_SetScreenSize(display_width, display_height);
}

void display_on_mouse(void)
{
    /* Change mouse display on */
    g_display0.p_api->start(g_display0.p_ctrl);
}

void display_off_mouse(void)
{
    /* Change mouse display off */
    g_display0.p_api->stop(g_display0.p_ctrl);
}

void change_mouse_position(uint16_t mouse_x, uint16_t mouse_y, __attribute__((unused)) uint16_t display_hw, __attribute__((unused)) uint16_t display_vw)
{
    /* Change mouse position data */
    g_display0_runtime_cfg_fg.input.coordinate_x = mouse_x;
    g_display0_runtime_cfg_fg.input.coordinate_y = mouse_y;

    /* change display of layer2 */
    g_display_on_lcdc.layerChange(g_display0.p_ctrl, &(g_display0_runtime_cfg_fg) , DISPLAY_FRAME_LAYER_2 );
}

r_usb_mouse_info_t usb_mouse_info;

/**************************************************************************//**
* Function Name: TP_SetPositionUsbMouse
* @brief         Set position information of USB mouse.
*
*                Description:<br>
*
* @param         [out]TP_TouchFinger_st *psTouch        : pointer to output table
* @param         [out]TPEVT_COORDINATES *psLastCoords   : pointer to last coordinate
* @param         [out]int_t *pnPenStatus                : pointer to pending status
* @param         [in]uint8_t *puReadBuf                 : pointer to receive buffer
* @param         [in]uint32_t drvnum                    : number of USB driver number
* @retval        None.
******************************************************************************/
void TP_SetPositionUsbMouse( TP_TouchFinger_st *psTouch, TPEVT_COORDINATES *psLastCoords, int32_t *pnPenStatus, const uint8_t* const puReadBuf)
{
    int16_t                  nXin;
    int16_t                  nYin;
    int32_t                  nXout;
    int32_t                  nYout;

    int32_t                  tmpPosX;
    int32_t                  tmpPosY;

    if (InputMode == TP_USE_TOUCH)
    {
		TP_GetPositionData(puReadBuf, &tmpPosX, &tmpPosY, usb_touch_info.data_len_pos,
						   usb_touch_info.data_offset_posX1, usb_touch_info.data_offset_posY1);

		// scale to resolution of screen
		int32_t scaledX = (tmpPosX * usb_touch_info.dev_max_posX) / usb_touch_info.dev_resolutionX;
		int32_t scaledY = (tmpPosY * usb_touch_info.dev_max_posY) / usb_touch_info.dev_resolutionY;
		nXin = (int16_t)scaledX;
		nYin = (int16_t)scaledY;

		// clamp to screen size
		TP_LineUpPos( nXin, nYin, &nXout, &nYout );

		psTouch->unPosX = (uint16_t)nXout;
		psTouch->unPosY = (uint16_t)nYout;

		/* state initialize */
		psTouch->eState = TPEVT_ENTRY_NONE;

		/* mouse moving */
		TP_EventHandleUsbMouse( &psTouch->eState, &nXout, &nYout, psLastCoords );

		static uint8_t laststate = 0;
		uint8_t state = puReadBuf[1];
		if (state != 0)
		{
			if (state == laststate)
				psTouch->eState = TPEVT_ENTRY_MOVE;
			else
				psTouch->eState = TPEVT_ENTRY_DOWN;
		}
		else
		{
			psTouch->eState = TPEVT_ENTRY_UP;
		}
		laststate = state;
    }
    else if (InputMode == TP_USE_MOUSE)
    {
		TP_GetPositionData(puReadBuf, &tmpPosX, &tmpPosY, usb_mouse_info.data_len_pos,
						   usb_mouse_info.data_offset_posX1, usb_mouse_info.data_offset_posY1);

		/* calculate coordinates from mouse movement */
		if (USB_POS_TYPE_RELATIVE == usb_mouse_info.pos_data_type)
		{
			nXin = (int16_t)(psTouch->unPosX + (tmpPosX * (int32_t)usb_mouse_info.cursor_speed) );
			nYin = (int16_t)(psTouch->unPosY + (tmpPosY * (int32_t)usb_mouse_info.cursor_speed) );
		}
		else
		{
			nXin = (int16_t)(psTouch->unPosX - (tmpPosX * (int32_t)usb_mouse_info.cursor_speed) );
			nYin = (int16_t)(psTouch->unPosY - (tmpPosY * (int32_t)usb_mouse_info.cursor_speed) );
		}

		TP_LineUpPos( nXin, nYin, &nXout, &nYout );

		psTouch->unPosX = (uint16_t)nXout;
		psTouch->unPosY = (uint16_t)nYout;

		/* state initialize */
		psTouch->eState = TPEVT_ENTRY_NONE;

		/* mouse moving */
		TP_EventHandleUsbMouse( &psTouch->eState, &nXout, &nYout, psLastCoords );

		if( *pnPenStatus == TP_PENSTATUS_PUSH )
		{
			if (0x00 == (puReadBuf[usb_mouse_info.data_offset_click] & usb_mouse_info.left_click_bit))
			{
				/* release left click, EVENT : UP */
				*pnPenStatus    = TP_PENSTATUS_RELEASE;        /* Touch Panel pen status, 0 : up */
				psTouch->eState = TPEVT_ENTRY_UP;
			}
			else
			{
				/* hold left click, EVENT : MOVE */
				psTouch->eState = TPEVT_ENTRY_MOVE;
			}
		}
		else
		{
			if (0x01 == (puReadBuf[usb_mouse_info.data_offset_click] & usb_mouse_info.left_click_bit))
			{
				/* push left click, EVENT : DOWN */
				*pnPenStatus    = TP_PENSTATUS_PUSH;        /* Touch Panel pen status, 1 : down */
				psTouch->eState = TPEVT_ENTRY_DOWN;
			}
		}
    }
}

/*Get position data from USB Buffer*/
static void TP_GetPositionData(const uint8_t* const puReadBuf, int32_t *PosX, int32_t *PosY, const uint32_t data_len, uint32_t posx_offset, uint32_t posy_offset )
{
    if (1 == data_len)
    {
        /* get 8bit X axis coordinate data, it's a data such as mouse mainly */
        *PosX = 0 + (int8_t)puReadBuf[posx_offset];
    }
    else
    {
        /* get 16bit X axis coordinate data, it's a data such as touch mainly */
        *PosX = 0 + (int32_t)(((uint16_t)puReadBuf[posx_offset + 1] << 8) + puReadBuf[posx_offset]);
    }

    if (1 == data_len)
    {
        /* get 8bit Y axis coordinate data, it's a data such as mouse mainly */
        *PosY = 0 + (int8_t)puReadBuf[posy_offset];
    }
    else
    {
        /* get 16bit Y axis coordinate data, it's a data such as touch mainly */
        *PosY = 0 + (int32_t)(((uint16_t)puReadBuf[posy_offset + 1] << 8) + puReadBuf[posy_offset]);
    }

}
static void TP_LineUpPos( const int16_t nXin, const int16_t nYin, int32_t *pnXout, int32_t *pnYout )
{
    int32_t   nScrWidth;
    int32_t   nScrHeight;

    TP_GetScreenSize( &nScrWidth, &nScrHeight );
    *pnXout = nXin;
    *pnYout = nYin;

    //DBG_printf_DBG( "[TPDBG] S(%d:%d), I(%d:%d), O(%d:%d)\n", nScrWidth, nScrHeight, nXin, nYin, *pnXout, *pnYout );
    if(*pnXout < 0)
    {
        *pnXout = 0 ;
    }
    else if(*pnXout >= nScrWidth)
    {
        *pnXout = nScrWidth - 1 ;
    }
    else
    {
        /* do nothing */
    }

    if(*pnYout < 0)
    {
        *pnYout = 0 ;
    }
    else if (*pnYout >= nScrHeight)
    {
        *pnYout = nScrHeight - 1 ;
    }
    else
    {
        /* do nothing */
    }
}
int32_t TP_EventHandleUsbMouse( __attribute__((unused))TpEvt_EntryType *eEvent, const int32_t* const nPosX, const int32_t* const nPosY, TPEVT_COORDINATES *psLastCoords )
{
    int32_t                    nRet ;
    int32_t                    nOver ;
    TPEVT_COORDINATES        Coords ;
    int32_t   nScrWidth;
    int32_t   nScrHeight;

    nRet    = 0;

    Coords.x = *nPosX ;
    Coords.y = *nPosY ;

    nOver = TP_CheckDiffMin( psLastCoords, &Coords, TPEVT_MOVE_MINTHD_USB ) ;
    if( 0 != nOver )
    {
        /* display on mouse cursor and change mouse position. */
        psLastCoords->x = *nPosX ;
        psLastCoords->y = *nPosY ;
        TP_GetScreenSize( &nScrWidth, &nScrHeight );
        change_mouse_position((uint16_t)psLastCoords->x, (uint16_t)psLastCoords->y, (uint16_t)nScrWidth, (uint16_t)nScrHeight);
    }
    else
    {
        /* display on mouse cursor. */
        display_on_mouse();
    }

    return nRet ;
}

int32_t TP_CheckDiffMin( const TPEVT_COORDINATES* const psPox_1, const TPEVT_COORDINATES* const psPox_2, const uint32_t unThreshold )
{
    int32_t     nX, nY;
    uint32_t    unData;

    nX = psPox_1->x - psPox_2->x ;
    nY = psPox_1->y - psPox_2->y ;
    unData = ( uint32_t )( nX * nX ) + ( uint32_t )( nY * nY ) ;

    return(( unData > unThreshold ) ? 1 : 0 ) ;
}

void TP_GetScreenSize( int32_t *pnWidth, int32_t *pnHeight )
{
    *pnWidth  = ScreenWidth;
    *pnHeight = ScreenHeight;
}

void TP_SetScreenSize( int32_t pnWidth, int32_t pnHeight )
{
    ScreenWidth = pnWidth;
    ScreenHeight = pnHeight;
}


