/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics Corporation and/or its affiliates and may only
 * be used with products of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.
 * Renesas products are sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for
 * the selection and use of Renesas products and Renesas assumes no liability.  No license, express or implied, to any
 * intellectual property right is granted by Renesas.  This software is protected under all applicable laws, including
 * copyright laws. Renesas reserves the right to change or discontinue this software and/or this documentation.
 * THE SOFTWARE AND DOCUMENTATION IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND
 * TO THE FULLEST EXTENT PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY,
 * INCLUDING WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE
 * SOFTWARE OR DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR
 * DOCUMENTATION (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER,
 * INCLUDING, WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY
 * LOST PROFITS, OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/

#include "rza3ul_smarc_vcom.h"
#include <stdarg.h>
#include "FreeRTOS.h"
#include "task.h"

int vfprintf_custom(void *out, const char *format, va_list args);
static volatile uint32_t uart_wait_flag = 0;

/**
 * @name       uart_master_callback
 * @brief      Callback for uart channel module
 * @param[in]  *p_args : uart master callback data
 * @return     None
 */
static void uart_callback (uart_callback_args_t * const p_arg)
{
    if (p_arg->event & UART_EVENT_TX_COMPLETE)
    {
        uart_wait_flag = 1;
    }
}

/**
 * @brief      Open and configure the uart channel
 * @param[in]  channel : The desired channel to be configurated
 * @return     bsp_status_t : Error code (see #ErrorCode enumeration in )
 */
void bsp_vcom_open(void)
{
    g_uart0.p_api->open(g_uart0.p_ctrl, g_uart0.p_cfg);
    g_uart0.p_api->callbackSet(g_uart0.p_ctrl, uart_callback, NULL, NULL);
}

/**
 * @brief      printf
 * @param[in]  str
 * @return     none
 */
void bsp_vcom_print( char * str )
{
    static char data[256];
    uint32_t r;

    r = (uint32_t)strlen( str );

    if(r>256)
    {
        r = 256;
    }

    sprintf( data, str );

    uart_wait_flag = 0;
    g_uart0.p_api->write(g_uart0.p_ctrl, (uint8_t *)data, r);

    while (uart_wait_flag == 0)
    {
        vTaskDelay(1);
    }
}
