#include <stdlib.h>
#include "FreeRTOSConfig.h"
#include "FreeRTOS.h"
#include "portmacro.h"
#include "semphr.h"
#include "touch_gslX680.h"
#include "rza3ul_smarc_mouse.h"
#include "r_usb_basic.h"
#include "r_usb_hhid.h"
#include "r_usb_hhid_driver.h"
#include "r_usb_hhid_api.h"
#include "vector_usb_data.h"
#include "bsp_irq_id.h"
#include "hal_data.h"
#include "r_usb_basic_if.h"
#include <stdio.h>

#define SET_PROTOCOL            (USB_HID_SET_PROTOCOL | USB_HOST_TO_DEV | USB_CLASS | USB_INTERFACE)
#define BOOT_PROTOCOL           (0)
#define BUFSIZE                 (4*1024)

#define USB_FS_DEVICE_ADDRESS_1 (1)
#define TP_TOUCHNUM_MAX         (1)
#define USB_BUF_LEN             (128)

#define MOVE_REDUCE_VALUE       (0)

static  uint8_t     g_buf[8] USB_BUFFER_PLACE_IN_SECTION; /* USB Receive data */
static  uint8_t     g_hid_type;
static  uint32_t    g_setup_data;
static  uint16_t    g_mxps;
static  usb_event_info_t    g_event_info;

//static  uint8_t     g_store_buf[BUFSIZE] USB_BUFFER_PLACE_IN_SECTION;

volatile uint8_t    g_suspend_flug = 0;

static const usb_hhid_api_t g_hhid_on_usb =
{
    .typeGet            = R_USB_HHID_TypeGet,
    .maxPacketSizeGet   = R_USB_HHID_MaxPacketSizeGet,
};

static void     set_protocol (usb_instance_ctrl_t *p_ctrl, uint8_t protocol, uint8_t device_address);
void touch_task(void *pvParameters);
void R_BSP_WarmStart(bsp_warm_start_event_t event);
void TP_SetPositionUsbMouse( TP_TouchFinger_st *psTouch, TPEVT_COORDINATES *psLastCoords, int32_t *pnPenStatus, const uint8_t* const puReadBuf);
void touchevent_callback(touchevent_data_t* pkTouchData);
void GetTaskInfo(__attribute__((unused))void *pvParameters);

/******************************************************************************
 * Function Name   : set_protocol
 * Description     : Sending SetProtocol request to HID device
 * Arguments       : usb_ctrl_t   *p_ctrl : Pointer to usb_instance_ctrl_t structure.
 *                 : uint8_t      ptorocol: Protocol Type
 *                 : uint8_t      device_address: Device address that sends this request
 * Return value    : none
 ******************************************************************************/
static void set_protocol (usb_instance_ctrl_t *p_ctrl, uint8_t protocol, uint8_t device_address)
{
    usb_setup_t     setup;

    setup.request_type      = SET_PROTOCOL; /* bRequestCode:SET_PROTOCOL, bmRequestType */
    setup.request_value     = protocol;     /* wValue: Protocol Type */
    setup.request_index     = 0x0000;       /* wIndex:Interface */
    setup.request_length    = 0x0000;       /* wLength:Zero */
    g_usb_on_usb.hostControlTransfer(p_ctrl, &setup, (uint8_t *)&g_setup_data, device_address); /* Request Control transfer */
} /* End of function set_protocol */


void usb_apl_callback (usb_event_info_t *p_api_event, usb_hdl_t cur_task, usb_onoff_t usb_state)
{
    (void)usb_state;
    (void)cur_task;

    g_event_info = *p_api_event;
} /* End of function usb_apl_callback */


/* pvParameters contains TaskHandle_t */
void touch_task(__attribute__((unused))void *pvParameters)
{

    fsp_err_t err;

    int32_t             nPenStatus[TP_TOUCHNUM_MAX];

    TP_TouchEvent_st    sTouchEvt;
    TPEVT_COORDINATES   sLastCoords[TP_TOUCHNUM_MAX];
    touchevent_data_t   kTouchData = {0,0,TOUCH_EVENT_NONE};
    touchevent_data_t   kTouchData_back = kTouchData;
    uint32_t            move_reduce = MOVE_REDUCE_VALUE;
    memset( &sTouchEvt, 0, sizeof(TP_TouchEvent_st) );
    printf("touch_task start\n\r");


    R_BSP_WarmStart(BSP_WARM_START_POST_C);

    err = g_basic0.p_api->open(g_basic0.p_ctrl, g_basic0.p_cfg);
    if(FSP_SUCCESS != err)
    {
        while(1);
    }
    printf("USB opened\n\r");

    while (1)
    {
        /* Scheduler */
        R_USB_CstdScheduler();

        /* Check application state */
        switch (g_event_info.event)
        {
            case USB_STATUS_CONFIGURED :
            {
                g_hhid_on_usb.typeGet(&g_basic0_ctrl, &g_hid_type, g_event_info.device_address);
                g_hhid_on_usb.maxPacketSizeGet(&g_basic0_ctrl, &g_mxps, USB_HID_IN, g_event_info.device_address);

                    /* Send the HID request(SetProtocol) to HID device */
                set_protocol(&g_basic0_ctrl, BOOT_PROTOCOL, g_event_info.device_address);
                g_event_info.event = 1;
            break;
            }

            case USB_STATUS_READ_COMPLETE :
            {
                g_usb_on_usb.read(&g_basic0_ctrl, g_buf, (uint32_t)g_mxps, g_event_info.device_address);
                g_event_info.event = 1;
                TP_SetPositionUsbMouse(&sTouchEvt.sFinger[0], &sLastCoords[0], &nPenStatus[0], &g_buf[0]);
                kTouchData.x = sTouchEvt.sFinger[0].unPosX;
                kTouchData.y = sTouchEvt.sFinger[0].unPosY;
                switch(sTouchEvt.sFinger[0].eState)
                {
                    case TPEVT_ENTRY_UP:
                        kTouchData.event = TOUCH_EVENT_UP;
                        break;
                    case TPEVT_ENTRY_DOWN:
                        kTouchData.event = TOUCH_EVENT_DOWN;
                        break;
                    case TPEVT_ENTRY_MOVE:
                        kTouchData.event = TOUCH_EVENT_MOVE;
                        break;
                    default:
                        kTouchData.event = TOUCH_EVENT_NONE;
                        break;
                }
                if (kTouchData.event != TOUCH_EVENT_NONE)
                {
                    if( 0 == memcmp(&kTouchData, &kTouchData_back, sizeof(kTouchData) ) )
                    {
                        vTaskDelay(1);
                        if (0 < move_reduce)
                        {
                            move_reduce--;
                        }
                    }
                    else if ((TPEVT_ENTRY_MOVE == kTouchData_back.event) && (TPEVT_ENTRY_MOVE == kTouchData.event) && (0 < move_reduce))
                    {
                        vTaskDelay(1);
                        if (0 < move_reduce)
                        {
                            move_reduce--;
                        }
                    }
                    else
                    {
                        touchevent_callback(&kTouchData);
                        kTouchData_back = kTouchData;
                        move_reduce = MOVE_REDUCE_VALUE;
                    }
                }
                
            break;
            }
            
            case USB_STATUS_REQUEST_COMPLETE :
            {
                if (USB_HID_SET_PROTOCOL == (g_event_info.setup.request_type & USB_BREQUEST))
                {
                    g_usb_on_usb.read(&g_basic0_ctrl, g_buf, (uint32_t)g_mxps, g_event_info.device_address);
                    g_event_info.event = 1;
                }

            break;
            }

            default :
            {
                vTaskDelay(1);
            break;
            }
	    }


	}
}

