#include "main_thread.h"
#include "common_data.h"
#include "r_display_api.h"
#include "r_lcdc.h"

#include "arrow_data.h"

extern "C"
{
#include "hal_data.h"
#include <stdio.h>
#include <stdarg.h>
#include "rza3ul_smarc_lcd.h"
#include "rza3ul_smarc_vcom.h"
#include "touch_gslX680.h"
#include "adv7513.h"
#include "rza3ul_smarc_mouse.h"
}

/** Common macro for FSP header files. There is also a corresponding FSP_FOOTER macro at the end of this file. */
FSP_HEADER

extern void touch_task(void *pvParameter);
#define LCD_WIDTH  (800)
#define LCD_HEIGHT (480)

#if defined(GFX_USE_RGB565)
#define PIXELACCESS uint16_t
#define BLUE 0x001f
#define GREEN 0x07e0
#elif defined(GFX_USE_ARGB32)
#define PIxELACCESS uint32_t
#define BLUE 0xff0000ff
#define GREEN 0xff00ff00
#else
#error "please specify pixel-format"
#endif

#define USB_BUF_LEN                         (128)

/*************************************************************************
 Structures
*************************************************************************/


int global_x = 0;
int global_y = 0;
PIXELACCESS current_color = BLUE;
static char const * const event_string[] =
{
	"NONE",
	"DOWN",
	"HOLD",
	"MOVE",
	"UP"
};
static volatile PIXELACCESS* current_edit_buffer;
touchevent_data_t touch_event;

extern "C"
{
    void touchevent_callback(touchevent_data_t* pkTouchData);
    void DrawClear(unsigned int color);
    void DrawDot(int x, int y, unsigned int color);
    void DrawHLine (int x, int y, int length, unsigned int color);
    void DrawVLine (int x, int y, int length, unsigned int color);
    void initialise_monitor_handles (void);
    void TP_SetScreenSize( int32_t pnWidth, int32_t pnHeight );

    void touchevent_callback(touchevent_data_t* pkTouchData)
    {
        int x = pkTouchData->x;
        int y = pkTouchData->y;

        if (pkTouchData->event == TOUCH_EVENT_UP)
        {
            global_x = 0;
            global_y = 0;
        }
        else
        {
            global_x = x;
            global_y = y;

            DrawDot(global_x, global_y, 0xFFFFFFFFuL);
        }
        printf("(%03d,%03d):%s\n\r",x,y,event_string[pkTouchData->event]);
    }
    void DrawClear(unsigned int color)
    {
        int y;
        for (y = 0; y < LCD_HEIGHT; y++)
        {
            DrawHLine (0, y, LCD_WIDTH, color);
        }
    }

    void DrawDot(int x, int y, unsigned int color)
    {
        if (
            (x >= 0 && x < LCD_WIDTH) &&
            (y >= 0 && y < LCD_HEIGHT)
            )
        {
            current_edit_buffer[(y * LCD_WIDTH) + x ] = (PIXELACCESS)color;
        }
    }

    void DrawHLine (int x, int y, int length, unsigned int color)
    {
        // Verify starting point of horizontal line is within LCD range
        if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
        {
            // If ending point is over LCD range, reduce line length to fit LCD range
            if((x+length) > LCD_WIDTH)
            {
                length = LCD_WIDTH - x;
            }
            // Draw line
            for(uint16_t i = 0; i <= length; i++)
            {
                DrawDot(x, y, color);
                x++;
            }
        }
    }

    void DrawVLine (int x, int y, int length, unsigned int color)
    {
        // Verify starting point of vertical line is within LCD range
        if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
        {
            // If ending point is over LCD range, reduce line length to fit LCD range
            if((y+length) > LCD_HEIGHT)
            {
                length = LCD_HEIGHT - y;
            }
            // Draw line
            for(uint16_t i = 0; i < length; i++)
            {
                DrawDot(x, y, color);
                y++;
            }
        }
    }
    
}

/* MainTask entry function */
__attribute((used)) void software_reset(void)
{
    __disable_fiq();
    R_WDT0->WDTCNT = 0;
    R_WDT0->WDTSET = 0x00100000;
    R_WDT0->WDTTIM = 0;
    R_CPG->CPG_WDTRST_SEL = 0x00010001;
    R_WDT0->WDTINT = 1;
    R_WDT0->WDTCNT = 1;
    while(1);

}

void main_thread_entry(void *pvParameters)
{
    portBASE_TYPE status;
    TaskHandle_t task;

    uint16_t x;
    uint16_t y;
    uint16_t hw;
    uint16_t vw;

    if( R_WDT0->WDTCNT == 1)
    {
        software_reset();
    }
    //init vertical related mouse moving
    x = 0;
    y = 0;
    hw = LCD_WIDTH;
    vw = LCD_HEIGHT;


    FSP_PARAMETER_NOT_USED (pvParameters);
    initialise_monitor_handles();

    /* Configure pins. */
    R_IOPORT_Open(&g_ioport_ctrl, &g_bsp_pin_cfg);

    // Initialize uart
    printf((char*)"Initialize uart.\r\n");

    // Initialize buffer for lcdc
    current_edit_buffer = (PIXELACCESS*)&fb_background[0];
    current_color = BLUE;
    DrawClear(current_color);
    current_edit_buffer = (PIXELACCESS*)&fb_background[1];

    /* Init layer2 buffer */
#if 0
    pbuff_32 = (uint32_t *)fb_foreground[0];
    src_pbuff_32 = (uint32_t *)&g_arrow_data[0];
    for( height = 0 ; height < DISPLAY_VSIZE_INPUT0 ; height++ )
    {
        for( width = 0 ; width < DISPLAY_HSIZE_INPUT0 ; width++ )
        {
            pbuff_32[ DISPLAY_HSIZE_INPUT0 * height + width] = src_pbuff_32[DISPLAY_HSIZE_INPUT0 * height + width]; // ARGB8888
        }
    }
#endif
    TP_SetScreenSize(LCD_WIDTH,LCD_HEIGHT);

    //Create Touch task
    status = xTaskCreate(touch_task, "Touch Task", 8192, NULL, 30, &task);

    adv7513_init();
    printf((char*)"Initialize adv7513.\r\n");


    // Initialize lcdc
    bsp_lcd_init();
    bsp_lcd_display_on();
    printf((char*)"Initialize lcdc.\r\n");

    mouse_init(1280,800);
    display_off_mouse();
    display_on_mouse();
    
    while (1)
    {

        x++;
        y++;

        if( current_color == BLUE )
        {
            current_color = GREEN;
            printf((char*)"green.\r\n");
        }
        else
        {
            current_color = BLUE;
            printf((char*)"blue.\r\n");
        }
        DrawClear(current_color);

        // Change buffer
        current_edit_buffer = (PIXELACCESS*) bsp_lcd_display_setframeBuffer((void*)current_edit_buffer, 0, 0);


        vTaskDelay(1000);
    }
}

/** Common macro for FSP header files. There is also a corresponding FSP_HEADER macro at the top of this file. */
FSP_FOOTER
