
#
# Windows-MSVC-AMD64 specific settings
#

# Set output dirs
SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES RUNTIME_OUTPUT_DIRECTORY "${PROJECT_SOURCE_DIR}/Binary/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}")

IF(STREAMRUNTIME_SDK_PROJECT)
  TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}"
    PRIVATE "${PATH_GUILIANI}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}"
  )
ENDIF(STREAMRUNTIME_SDK_PROJECT)

# Set compiler parameters for VS
SET(CMAKE_CXX_FLAGS_DEBUGFIXPOINT "${CMAKE_CXX_FLAGS_DEBUG}")
SET(CMAKE_CXX_FLAGS_RELEASEFIXPOINT "${CMAKE_CXX_FLAGS_RELEASE}")

# Same as for C++
SET(CMAKE_C_FLAGS_DEBUGEFIXPOINT "${CMAKE_CXX_FLAGS_DEBUGFIXPOINT}")
SET(CMAKE_C_FLAGS_RELEASEFIXPOINT "${CMAKE_CXX_FLAGS_RELEASEFIXPOINT}")

SET(CMAKE_EXE_LINKER_FLAGS_DEBUG "${CMAKE_EXE_LINKER_FLAGS_DEBUG} /LIBPATH:\"${PROJECT_SOURCE_DIR}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}\"")
SET(CMAKE_EXE_LINKER_FLAGS_RELEASE "${CMAKE_EXE_LINKER_FLAGS_RELEASE} /LIBPATH:\"${PROJECT_SOURCE_DIR}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}\"")
SET(CMAKE_EXE_LINKER_FLAGS_DEBUGFIXPOINT "${CMAKE_EXE_LINKER_FLAGS_DEBUG}")
SET(CMAKE_EXE_LINKER_FLAGS_RELEASEFIXPOINT "${CMAKE_EXE_LINKER_FLAGS_RELEASE}")

# Includes - SDK independent
TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${PATH_GUILIANI}/Include/Platform/win"
  PRIVATE "${PATH_GUILIANI}/Resources/win/pc"
  PRIVATE "${PROJECT_SOURCE_DIR}/Common/Include/Platform/win/pc"
)

# use cli + shortcuts on this platform
LIST(APPEND COMPLEX_DEFINES "STREAMRUNTIME_USE_CLI;STREAMRUNTIME_USE_SHORTCUTS")

# Includes - SDK dependent
IF(STREAMRUNTIME_SDK_PROJECT)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/Include/eC/x86_win32"
  )
ELSE()
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/comps/eC/code/x86_win32"
  )
ENDIF()

FILE(GLOB PLATFORM_WIN_PC_CPP 
    "${PROJECT_SOURCE_DIR}/Common/Source/Platform/win/pc/new_delete.cpp" 
    "${PROJECT_SOURCE_DIR}/Common/Source/Platform/win/pc/StreamRuntime.cpp" 
    "${PROJECT_SOURCE_DIR}/Common/Source/Platform/win/pc/StreamRuntimeStartup.cpp"
    )
SOURCE_GROUP("Source Files\\Platform\\Windows\\PC" FILES ${PLATFORM_WIN_PC_CPP})

FILE(GLOB PLATFORM_WIN_PC_H 
    "${PROJECT_SOURCE_DIR}/Common/Include/Platform/win/pc/*.h"
    "${PATH_GUILIANI}/Resources/win/pc/*.h")
SOURCE_GROUP("Header Files\\Platform\\Windows\\PC" FILES ${PLATFORM_WIN_PC_H})

SET(RESOURCE_FILE "${PATH_GUILIANI}/Resources/win/pc/StreamRuntime.rc")
SOURCE_GROUP("Resource Files" FILES ${RESOURCE_FILE})

SET(PLATFORM_FILES
  ${PLATFORM_WIN_PC_CPP}
  ${PLATFORM_WIN_PC_H}
  ${RESOURCE_FILE}
)
  
# binary dependencies
SET(PLATFORM_LIBS
  "winmm.lib"
    )

#BLU
IF(GUILIANI_ADD_BLU_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_BLU}/Include"
  )
ENDIF(GUILIANI_ADD_BLU_PLATFORM)

#GlyphLib
IF(GUILIANI_ADD_GLYPHLIB_PLATFORM)
  TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_GLYPHLIB}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}"
    )

  LIST(APPEND PLATFORM_LIBS
    "GlyphLib.lib"
  )
ENDIF(GUILIANI_ADD_GLYPHLIB_PLATFORM)

IF(NOT EGAC_BUILD_IMAGELIBS)
  IF(NOT LIBS_BUILD_LIBPNG)
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
	  PRIVATE "${PATH_LIBS}/png/lib"
    )

    IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "libpng16.lib"
      )
    ELSE("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "libpng16_x64.lib"
      )
    ENDIF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
  ENDIF(NOT LIBS_BUILD_LIBPNG)

  IF(NOT LIBS_BUILD_LIBJPEG)
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
	  PRIVATE "${PATH_LIBS}/jpeg/lib"
    )

    IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "libjpeg6b.lib"
      )
    ELSE("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "libjpeg6b_x64.lib"
      )
    ENDIF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
  ENDIF(NOT LIBS_BUILD_LIBJPEG)

  IF(NOT LIBS_BUILD_ZLIB)
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
	  PRIVATE "${PATH_LIBS}/zlib/lib"
    )

    IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "zlib1211.lib"
      )
    ELSE("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      LIST(APPEND PLATFORM_LIBS
        "zlib1211_x64.lib"
      )
    ENDIF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
  ENDIF(NOT LIBS_BUILD_ZLIB)
ENDIF(NOT EGAC_BUILD_IMAGELIBS)

# ogl
IF(GUILIANI_ADD_GL_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/opengl/include"
  )

  LIST(APPEND PLATFORM_LIBS 
    "opengl32.lib"
    "glu32.lib"
  )
ENDIF()

#DAVE
IF(GUILIANI_ADD_DAVE_PLATFORM)
  IF(STREAMRUNTIME_SDK_PROJECT)
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_DAVE}/Include"
      PRIVATE "${PATH_DAVE}/Include/d0lib"
      PRIVATE "${PATH_DAVE}/Include/softdave"
    )
  ELSE()
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include"
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include/d0lib"
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include/softdave"
    )
  ENDIF()

  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/DAVE/win32"
    )
  ELSE()
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/DAVE/win64"
    )
  ENDIF()
  
  LIST(APPEND PLATFORM_LIBS
    "davedll.lib"
    "libtcc.lib"
  )
ENDIF(GUILIANI_ADD_DAVE_PLATFORM)

#DAVE HD
IF(GUILIANI_ADD_DAVEHD_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/DAVEHD/win"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/config"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/config/platform"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/kernel"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/user"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/softdave"
  )
    
  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/DAVEHD/win32"
    )
  ELSE()
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/DAVEHD/win64"
    )
  ENDIF()
  
  LIST(APPEND PLATFORM_LIBS
    "davehd_sd.lib"
    "dhd_kernel.lib"
    "dhd_user.lib"
  )
ENDIF(GUILIANI_ADD_DAVEHD_PLATFORM)

# Vulkan
IF(GUILIANI_ADD_VULKAN_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/Vulkan"
    PRIVATE "${PATH_VULKAN}/Include"
  )

  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_VULKAN}/Lib32"
    )
  ELSE()
    TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_VULKAN}/Lib"
    )
  ENDIF()

  LIST(APPEND PLATFORM_LIBS
    "vulkan-1.lib"
  )
ENDIF(GUILIANI_ADD_VULKAN_PLATFORM)

#FREETYPE
IF(LIBS_BUILD_FREETYPE)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_FREETYPE}/include" 
  )
ELSE(LIBS_BUILD_FREETYPE)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/freetype/include" 
    PRIVATE "${PATH_LIBS}/freetype/include/freetype" 
  )

  TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/freetype/lib"
  )
  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    LIST(APPEND PLATFORM_LIBS
      "freetype248.lib"
    )
  ELSE("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    LIST(APPEND PLATFORM_LIBS
      "freetype248_x64.lib"
    )
  ENDIF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
ENDIF(LIBS_BUILD_FREETYPE)

INCLUDE("${PROJECT_SOURCE_DIR}/CMake/PluginLoop.cmake")
