/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 ******************************************************************************/

#include <stdint.h>

#include "r_lcdc.h"

extern "C"
{
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"
#include "hal_data.h"
#include <stdio.h>
#include <stdarg.h>
#include "rza3ul_smarc_lcd.h"
#include "rza3ul_smarc_mouse.h"
#include "rza3ul_smarc_vcom.h"
#include "adv7513.h"
}

extern "C" void TP_SetScreenSize( int32_t pnWidth, int32_t pnHeight );
extern "C" void touch_task(void *pvParameter);

/******************************************************************************
 Private global variables and functions
 ******************************************************************************/

/******************************************************************************
 Imported global variables and functions (from other files)
 ******************************************************************************/

#define LCD_WIDTH DISPLAY_HSIZE_INPUT0
#define LCD_HEIGHT DISPLAY_VSIZE_INPUT0

#if defined(GFX_USE_RGB565)
    #define LCD_BYTES_PER_PIXEL 2
#elif defined(GFX_USE_RGB888)
    #define LCD_BYTES_PER_PIXEL 4
#else
    #error "choose color-format"
#endif

uint8_t* draw_buffer = NULL;
uint8_t* display_buffer = NULL;

extern uint8_t *RZAFrameBuffers[];

int draw_buffer_index = 0;
int display_buffer_index = 1;

uint32_t GetFrameBufferWidth()
{
	return LCD_WIDTH;
}

uint32_t GetFrameBufferHeight()
{
	return LCD_HEIGHT;
}

extern "C" void FlipFramebuffer()
{
    draw_buffer_index = (draw_buffer_index + 1) % 2;
    draw_buffer = RZAFrameBuffers[draw_buffer_index];

    display_buffer_index = (display_buffer_index + 1) % 2;
	display_buffer = RZAFrameBuffers[display_buffer_index];

	bsp_lcd_display_setframeBuffer(display_buffer, 0, 0);
}

extern "C" void CopyBuffer(int iX1, int iY1, int iRectWidth, int iRectHeight)
{
    uint8_t* pubSource = display_buffer + (iY1 * LCD_WIDTH + iX1) * LCD_BYTES_PER_PIXEL;
    uint8_t* pubDest = draw_buffer + (iY1 * LCD_WIDTH + iX1) * LCD_BYTES_PER_PIXEL;
    uint32_t pitch = LCD_WIDTH * LCD_BYTES_PER_PIXEL;
    for (uint16_t iLineIndex = 0; iLineIndex < iRectHeight; ++iLineIndex)
    {
        memcpy(pubDest, pubSource, iRectWidth * LCD_BYTES_PER_PIXEL);

        pubSource += pitch;
        pubDest += pitch;
    }
}

#if defined(GFX_USE_RGB565)
void DrawPixel(int x, int y, uint16_t color)
{
	uint16_t* buffer = (uint16_t*)draw_buffer + ((y * LCD_WIDTH) + x);
	*buffer = color;
}
#elif defined(GFX_USE_RGB888)
void DrawPixel(int x, int y, uint32_t color)
{
	uint32_t* buffer = (uint32_t*)draw_buffer + ((y * LCD_WIDTH) + x);
	*buffer = color;
}
#endif

#if defined(GFX_USE_RGB565)
void DrawVerLine(int x, uint16_t color)
#elif defined(GFX_USE_RGB888)
void DrawVerLine(int x, uint32_t color)
#endif
{
	for (int y = 0; y < LCD_HEIGHT; ++y)
	{
		DrawPixel(x, y, color);
	}
}

#if defined(GFX_USE_RGB565)
extern "C" void FillRect(int x, int y, int sizex, int sizey, uint16_t color)
#elif defined(GFX_USE_RGB888)
extern "C" void FillRect(int x, int y, int sizex, int sizey, uint32_t color)
#endif
{
	for (int cy = y; cy < y + sizey; ++cy)
	{
		for (int cx = x; cx < x + sizex; ++cx)
		{
			DrawPixel(cx, cy, color);
		}
	}
}

#if defined(GFX_USE_RGB565)
static const uint32_t colors[] =
{
	0x0000,
	0xf800,
	0x07e0,
	0x001f,
	0xffe0,
	0xf81f,
	0x07ff,
	0xffff
};
#elif defined(GFX_USE_RGB888)
static const uint32_t colors[] =
{
	0xff000000,
	0xffff0000,
	0xff00ff00,
	0xff0000ff,
	0xffffff00,
	0xffff00ff,
	0xff00ffff,
	0xffffffff
};
#endif

void GraphicTest()
{
    uint32_t color_index = 0;

    for (int x = 0; x < LCD_WIDTH; ++x)
    {
    	// draw vertical line
    	DrawVerLine(x, colors[color_index]);
    	color_index = (color_index + 1) % 8;
    }
    FlipFramebuffer();

#if 0
    while(1)
    {
        memset((void*)draw_buffer, color, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
        color += 0x12;

        FlipFramebuffer();

        vTaskDelay(50);
    }
#endif
}

void GraphicInit()
{
    draw_buffer_index = 0;
    draw_buffer = RZAFrameBuffers[draw_buffer_index];
    display_buffer_index = 1;
    display_buffer = RZAFrameBuffers[display_buffer_index];

    memset((void*)draw_buffer, 0xff, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
    memset((void*)display_buffer, 0x88, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);

    // Initialize adv7513 for HDMI output
    adv7513_init();

    // Initialize lcdc
    bsp_lcd_init();
    bsp_lcd_display_on();
}

void InputInit()
{
    //Create Touch task
    portBASE_TYPE status;
    TaskHandle_t task;
    status = xTaskCreate(touch_task, "Touch Task", 8192, NULL, 30, &task);
    if (status != pdPASS)
        bsp_vcom_print((char*)"Initialize touch failed.\r\n");

    // init mouse-pointer
    pointer_init();

    mouse_init(GetFrameBufferWidth(), GetFrameBufferHeight());

    // last two parameters are the resolution of the touch-device
	//touch_init(GetFrameBufferWidth(), GetFrameBufferHeight(), 0x3fff, 0x24ff);

    display_off_mouse();
    display_on_mouse();
}

void InitPeripherals()
{
    GraphicInit();

    InputInit();
}

extern "C"
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    extern volatile uint8_t ucHeap[ configTOTAL_HEAP_SIZE ];
#endif

void* GetHeapStart()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return (void*)&ucHeap[0];
#else
    #error "no heap allocated"
    return NULL;
#endif
}

void* GetHeapEnd()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return (void*)((uint8_t*)&ucHeap[0] + configTOTAL_HEAP_SIZE);
#else
    #error "no heap allocated"
    return NULL;
#endif
}

uint32_t GetHeapSize()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return configTOTAL_HEAP_SIZE;
#else
    return 0;
#endif
}

void PrintString(const char* message)
{
    printf(message);
}
}

const long resource_size = 0x00900000;

extern const uint8_t __attribute__ ((section (".RES_FLASH"))) __guilianiResources[resource_size];

const uint8_t* GetResources()
{
    return __guilianiResources;
}
