#include "GUIInputRZA3.h"

extern "C"
{
#include <ctype.h>
#include "FreeRTOS.h"
#include "task.h"
#include "touch_gslX680.h"
#include "adv7513.h"
#include "rza3ul_smarc_mouse.h"
}

#define INVALID_AXIS_VALUE -1

#include "eC_String.h"
#include "GUITimer.h"
#include "GUITrace.h"

#include "GUIMemLeakWatcher.h"
#include "WindowsLeakWatcher.h"

#define QUEUED_EVENTS 1
#define DEBUG_TOUCH_EVENTS 0

static CGUIInputRZA3* s_pkInputDevice = NULL;

extern "C"
{
// callback for touch-events
void touchevent_callback(touchevent_data_t* pkTouchData)
{
    if ((NULL != s_pkInputDevice) && (NULL != pkTouchData))
    {
        s_pkInputDevice->AddEvent(*pkTouchData);
    }
}
}

CGUIInputRZA3::CGUIInputRZA3(
    const eC_Int& iTouchScreenX,
    const eC_Int& iTouchScreenY,
    const eC_Int& iTouchScreenWidth,
    const eC_Int& iTouchScreenHeight,
    const eC_Int& iTouchFilterDelta) :
    CGUIInputMedia(false, CGUIInputMedia::Callback),
    m_iTouchScreenX(iTouchScreenX),
    m_iTouchScreenY(iTouchScreenY),
    m_iTouchScreenWidth(iTouchScreenWidth),
    m_iTouchScreenHeight(iTouchScreenHeight),
    m_iTouchFilterDelta(iTouchFilterDelta),
    m_iLastDraggedX(INVALID_AXIS_VALUE),
    m_iLastDraggedY(INVALID_AXIS_VALUE),
    m_iLastPositionX(INVALID_AXIS_VALUE),
    m_iLastPositionY(INVALID_AXIS_VALUE)
{
    s_pkInputDevice = this;
}

CGUIInputRZA3::~CGUIInputRZA3()
{
}

void CGUIInputRZA3::CreateInstance(
    const eC_Int& iTouchScreenX,
    const eC_Int& iTouchScreenY,
    const eC_Int& iTouchScreenWidth,
    const eC_Int& iTouchScreenHeight,
    const eC_Int& iTouchFilterDelta)
{
    if (!CGUIComponentManager::GetInstance().HasInputMedia())
        CGUIComponentManager::GetInstance().SetInputMedia(new CGUIInputRZA3(iTouchScreenX, iTouchScreenY, iTouchScreenWidth, iTouchScreenHeight, iTouchFilterDelta));
}

void CGUIInputRZA3::AddEvent(touchevent_data_t kTouchEvent)
{
    eC_Bool bCreateEvent = false;

    if (
        (m_eLastTouchEvent.event != kTouchEvent.event) ||
        (abs(m_iLastPositionX - kTouchEvent.x) > m_iTouchFilterDelta) ||
        (abs(m_iLastPositionY - kTouchEvent.y) > m_iTouchFilterDelta)
        )
    {
        if (
            (m_eLastTouchEvent.event == kTouchEvent.event) ||
            (kTouchEvent.event == TOUCH_EVENT_MOVE)
            )
        {
            // just a move
            kTouchEvent.event = TOUCH_EVENT_MOVE;
            m_iLastPositionX = kTouchEvent.x;
            m_iLastPositionY = kTouchEvent.y;
        }
        else
        {
            switch (kTouchEvent.event)
            {
                case TOUCH_EVENT_UP:
                    m_iLastDraggedX = m_iLastDraggedY = INVALID_AXIS_VALUE;
                    m_iLastPositionX = m_iLastPositionY = INVALID_AXIS_VALUE;
                    break;

                case TOUCH_EVENT_DOWN:
                    m_iLastPositionX = kTouchEvent.x;
                    m_iLastPositionY = kTouchEvent.y;
                    break;


                default:
                    break;
            }            
        }

        m_eLastTouchEvent = kTouchEvent;
        bCreateEvent = true;
    }

    if (true == bCreateEvent)
    {
        CGUIEvent* pkEvent = NULL;
        switch (kTouchEvent.event)
        {
            case TOUCH_EVENT_UP:
                pkEvent = new CGUIMouseEvent(ET_LBUTTONUP, kTouchEvent.x, kTouchEvent.y);
                break;

            case TOUCH_EVENT_DOWN:
                pkEvent = new CGUIMouseEvent(ET_LBUTTONDOWN, kTouchEvent.x, kTouchEvent.y, true);
                break;

            case TOUCH_EVENT_MOVE:
                pkEvent = new CGUIMouseEvent(ET_MOUSEMOVE, kTouchEvent.x, kTouchEvent.y, true, false);
                break;

            default:
                break;
        }

        CGUIInputMedia::AddEvent(pkEvent);
    }
}
