#include "DemoAdvanced.h"

#include "GUI.h"
#include "GUIDataPool.h"

/*Always include last*/
#include "GUIMemLeakWatcher.h"

#define USE_SMOOTH_ANIMATION 1

eC_String WeekDays_German[] =
{
    "So",
    "Mo",
    "Di",
    "Mi",
    "Do",
    "Fr",
    "Sa"
};

eC_String WeekDays_Russian[] =
{
    eC_String("\xD0\x92\xD0\xA1", 0, eC_String::eUTF8),
    eC_String("\xD0\x9F\xD0\x9E", 0, eC_String::eUTF8),
    eC_String("\xD0\x92\xD0\xA2", 0, eC_String::eUTF8),
    eC_String("\xD0\xA1\xD0\xA0", 0, eC_String::eUTF8),
    eC_String("\xD0\xA7\xD0\xA2", 0, eC_String::eUTF8),
    eC_String("\xD0\x9F\xD0\xA2", 0, eC_String::eUTF8),
    eC_String("\xD0\xA1\xD0\x91", 0, eC_String::eUTF8)
};

eC_String WeekDays_English[] =
{
    "Su",
    "Mo",
    "Di",
    "We",
    "Th",
    "Fr",
    "Sa"
};

eC_String Months_German[] =
{
    "Januar",
    "Februar",
    "Mrz",
    "April",
    "Mai",
    "Juni",
    "Juli",
    "August",
    "September",
    "Oktober",
    "November",
    "Dezember"
};

eC_String Months_Russian[] =
{
    eC_String("\xD0\xAF\xD0\xBD\xD0\xB2\xD0\xB0\xD1\x80\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\xA4\xD0\xB5\xD0\xB2\xD1\x80\xD0\xB0\xD0\xBB\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x9C\xD0\xB0\xD1\x80\xD1\x82", 0, eC_String::eUTF8),
    eC_String("\xD0\x90\xD0\xBF\xD1\x80\xD0\xB5\xD0\xBB\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x9C\xD0\xB0\xD0\xB9", 0, eC_String::eUTF8),
    eC_String("\xD0\x98\xD1\x8E\xD0\xBD\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x98\xD1\x8E\xD0\xBB\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x90\xD0\xB2\xD0\xB3\xD1\x83\xD1\x81\xD1\x82", 0, eC_String::eUTF8),
    eC_String("\xD0\xA1\xD0\xB5\xD0\xBD\xD1\x82\xD1\x8F\xD0\xB1\xD1\x80\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x9E\xD0\xBA\xD1\x82\xD1\x8F\xD0\xB1\xD1\x80\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x9D\xD0\xBE\xD1\x8F\xD0\xB1\xD1\x80\xD1\x8C", 0, eC_String::eUTF8),
    eC_String("\xD0\x94\xD0\xB5\xD0\xBA\xD0\xB0\xD0\xB1\xD1\x80\xD1\x8C", 0, eC_String::eUTF8)
};

eC_String Months_English[] =
{
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December"
};

CalendarObserver::CalendarObserver(CGUIBaseTextField* pkTargetObject) :
    m_pkTargetObject(pkTargetObject)
{
}

void CalendarObserver::OnNotification(const CGUIValue& kObservedValue, const CGUIObject* const pkUpdatedObject, const eC_UInt uiX, const eC_UInt uiY)
{
    if (NULL != m_pkTargetObject)
    {
        m_pkTargetObject->GetLabel()->SetTextString(kObservedValue.ToString());
        m_pkTargetObject->InvalidateArea();
    }
}

ClockObserver::ClockObserver(CGUIBaseTextField* pkTargetObject) :
    m_pkTargetObject(pkTargetObject)
{
}

void ClockObserver::OnNotification(const CGUIValue& kObservedValue, const CGUIObject* const pkUpdatedObject, const eC_UInt uiX, const eC_UInt uiY)
{
    if (NULL != m_pkTargetObject)
    {
        m_pkTargetObject->GetLabel()->SetTextString(kObservedValue.ToString());
        m_pkTargetObject->InvalidateArea();
    }
}

KnobObserver::KnobObserver(
    CGUIBaseTextField* pkTargetObject,
    CGUIKnob* pkKnob,
    CGUICircularSlider* pkCircularSlider) :
    m_pkTargetObject(pkTargetObject),
    m_pkKnob(pkKnob),
    m_pkCircularSlider(pkCircularSlider)
{
}

void KnobObserver::OnNotification(const CGUIValue& kObservedValue, const CGUIObject* const pkUpdatedObject, const eC_UInt uiX, const eC_UInt uiY)
{
    if (
        (NULL != m_pkTargetObject) &&
        (NULL != m_pkKnob) &&
        (NULL != m_pkCircularSlider)
        )
    {
        if (pkUpdatedObject == m_pkKnob)
        {
            // remove observer and re-attach to prevent recursion
            m_pkCircularSlider->RemoveValueObserver(this);
            m_pkCircularSlider->SetValue(kObservedValue.ToInt());
            m_pkCircularSlider->AddValueObserver(this);
        }
        else if (pkUpdatedObject == m_pkCircularSlider)
        {
            // remove observer and re-attach to prevent recursion
            m_pkKnob->RemoveValueObserver(this);
            m_pkKnob->SetValue(kObservedValue.ToInt());
            m_pkKnob->AddValueObserver(this);
        }

        // update textfield
        m_pkTargetObject->GetLabel()->SetTextString(kObservedValue.ToString());
        m_pkTargetObject->InvalidateArea();
    }
}

GaugeObserver::GaugeObserver(
    CGUIGauge* pkGauge,
    CGUIWheel* pkWheel) :
    m_pkGauge(pkGauge),
    m_pkWheel(pkWheel)
{
}

void GaugeObserver::OnNotification(const CGUIValue& kObservedValue, const CGUIObject* const pkUpdatedObject, const eC_UInt uiX, const eC_UInt uiY)
{
    if (
        (NULL != m_pkGauge) &&
        (NULL != m_pkWheel)
        )
    {
        if (pkUpdatedObject == m_pkGauge)
        {
            // remove observer and re-attach to prevent recursion
            m_pkWheel->RemoveValueObserver(this);
            m_pkWheel->SetValue(kObservedValue.ToInt());
            m_pkWheel->AddValueObserver(this);
        }
        else if (pkUpdatedObject == m_pkWheel)
        {
            // remove observer and re-attach to prevent recursion
            m_pkGauge->RemoveValueObserver(this);
            m_pkGauge->SetValue(kObservedValue.ToInt());
            m_pkGauge->AddValueObserver(this);
        }
    }
}

GaugeAnimation::GaugeAnimation(CGUIGauge* pkGauge, eC_Int iTargetValue) :
    CGUIAnimation(pkGauge, 2000, 100, false),
    m_pkGauge(pkGauge),
    m_iStartValue(0),
    m_iTargetValue(iTargetValue)
{
    if (NULL != m_pkGauge)
        m_iStartValue = m_pkGauge->GetRange().GetValue();
}

GaugeAnimation::~GaugeAnimation()
{
    m_pkGauge = NULL;
}

void GaugeAnimation::ApplyAnimation(eC_UInt uiElapsedTime)
{
    if (NULL != m_pkGauge)
    {
        CGUIEasing kEasing;
        eC_Int iValue = eC_ToInt(kEasing(CGUIEasing::EASE_LINEAR, eC_FromInt(uiElapsedTime), eC_FromInt(m_iStartValue), eC_FromInt(m_iTargetValue - m_iStartValue), eC_FromInt(2000)));
        m_pkGauge->SetValue(CGUIValue(iValue));
    }
}

DemoAdvanced::DemoAdvanced(
    CGUIEasing::EasingType_t eEasingUp,
    CGUIEasing::EasingType_t eEasingDown) :
    m_pkCalendar(NULL),
    m_pkCalendarTextField(NULL),
    m_pkCalendarObserver(NULL),
    m_pkClock(NULL),
    m_pkClockTextField(NULL),
    m_pkClockObserver(NULL),
    m_pkKnob(NULL),
    m_pkCircularSlider(NULL),
    m_pkKnobTextField(NULL),
    m_pkKnobObserver(NULL),
    m_vWheelValueFromDatapool(eC_FromInt(0)),
    m_vValueBuffer(eC_FromInt(0)),
    m_pkMainGauge(NULL),
    m_pkWheelControl(NULL),
    m_pkGaugeObserver(NULL),
    m_eEasingUp(eEasingUp),
    m_eEasingDown(eEasingDown),
    m_pkGaugeAnimation(NULL)
{
}

DemoAdvanced::~DemoAdvanced()
{
    DeInit();
}

void DemoAdvanced::Init(const eC_UInt& uiTime)
{
    m_pkDialog = auto_cast<CGUICompositeObject*>(GETGUI.GetObjectByID(DLG_ADVANCED));
    if (NULL != m_pkDialog)
    {
        m_pkCalendar = auto_cast<CGUICalendar*>(m_pkDialog->GetObjectByID(OBJ_CALENDAR));
        m_pkCalendarTextField = auto_cast<CGUIBaseTextField*>(m_pkDialog->GetObjectByID(OBJ_CALENDAR_TEXT));

        if ((NULL != m_pkCalendar) && (NULL != m_pkCalendarTextField))
        {
            m_pkCalendarObserver = new CalendarObserver(m_pkCalendarTextField);
            m_pkCalendar->AddValueObserver(m_pkCalendarObserver);
            m_pkCalendar->SetDate(17, 4, 2021);
        }

        m_pkClock = auto_cast<CGUIClock*>(m_pkDialog->GetObjectByID(OBJ_CLOCK));
        m_pkClockTextField = auto_cast<CGUIBaseTextField*>(m_pkDialog->GetObjectByID(OBJ_CLOCK_TEXT));

        if ((NULL != m_pkClock) && (NULL != m_pkClockTextField))
        {
            m_pkClockObserver = new ClockObserver(m_pkClockTextField);
            m_pkClock->AddValueObserver(m_pkClockObserver);
            m_pkClock->SetTimeInSeconds(uiTime);
        }

        m_pkKnob = auto_cast<CGUIKnob*>(m_pkDialog->GetObjectByID(OBJ_KNOB));
        m_pkCircularSlider = auto_cast<CGUICircularSlider*>(m_pkDialog->GetObjectByID(OBJ_CIRC_SLIDER));
        m_pkKnobTextField = auto_cast<CGUIBaseTextField*>(m_pkDialog->GetObjectByID(OBJ_CIRC_TEXT));

        if (
            (NULL != m_pkKnob) &&
            (NULL != m_pkKnobTextField) &&
            (NULL != m_pkCircularSlider)
            )
        {
            m_pkKnobObserver = new KnobObserver(m_pkKnobTextField, m_pkKnob, m_pkCircularSlider);
            m_pkKnob->AddValueObserver(m_pkKnobObserver);
            m_pkCircularSlider->AddValueObserver(m_pkKnobObserver);

            m_pkKnob->SetValue(-5);

#if defined(eC_TARGET_CPU_RA6M3) || defined(GFX_USE_BLU)
            // use primitives for the knob
            m_pkKnob->SetImages(IMG_KNOB_BACKGROUND, DUMMY_IMAGE);
#endif
        }

        m_pkMainGauge = auto_cast<CGUIGauge*>(m_pkDialog->GetObjectByID(GAUGE_ANIMATED));
#if defined(eC_TARGET_CPU_RA6M3) || defined(GFX_USE_BLU)
        // use pritmives for the gauge
        if (NULL != m_pkMainGauge)
            m_pkMainGauge->SetNeedleImage(DUMMY_IMAGE);
#endif
        m_pkWheelControl = auto_cast<CGUIWheel*>(m_pkDialog->GetObjectByID(VERTICAL_WHEEL));

        if (
            (NULL != m_pkMainGauge) &&
            (NULL != m_pkWheelControl)
            )
        {
            m_pkGaugeObserver = new GaugeObserver(m_pkMainGauge, m_pkWheelControl);
            m_pkMainGauge->AddValueObserver(m_pkGaugeObserver);
            m_pkWheelControl->AddValueObserver(m_pkGaugeObserver);

            m_pkKnob->SetValue(-5);
        }
    }
}

void DemoAdvanced::DeInit()
{
    delete m_pkCalendarObserver;
    m_pkCalendarObserver = NULL;

    delete m_pkClockObserver;
    m_pkClockObserver = NULL;

    delete m_pkKnobObserver;
    m_pkKnobObserver = NULL;
}

eC_UInt DemoAdvanced::GetTime() const
{
    if (m_pkClock)
        return m_pkClock->GetTimeInSeconds();
    else
        return 0;
}

void DemoAdvanced::SetLanguage(const eC_Int& iActiveLanguage)
{
    if (m_pkCalendar)
    {
        eC_UByte i;
        switch (iActiveLanguage)
        {
            case 1:
                for (i = 0; i < 7; ++i)
                    m_pkCalendar->SetDayText(i, WeekDays_German[i]);

                for (i = 0; i < 12; ++i)
                    m_pkCalendar->SetMonthText(i, Months_German[i]);
                break;

            case 2:
                for (i = 0; i < 7; ++i)
                    m_pkCalendar->SetDayText(i, WeekDays_Russian[i]);

                for (i = 0; i < 12; ++i)
                    m_pkCalendar->SetMonthText(i, Months_Russian[i]);
                break;

            default:
                for (i = 0; i < 7; ++i)
                    m_pkCalendar->SetDayText(i, WeekDays_English[i]);

                for (i = 0; i < 12; ++i)
                    m_pkCalendar->SetMonthText(i, Months_English[i]);
                break;
        }
    }
}

void DemoAdvanced::HandleCallAPI(const eC_String& kAPI, const eC_String kParam)
{
    if (kAPI == "ToggleKnob")
    {
        if (NULL != m_pkKnob)
        {
            m_pkKnob->SetAxisControl(!m_pkKnob->GetAxisControl());
        }
    }

    // Wheel
    if (kAPI == "HighWheel")
    {
#if USE_SMOOTH_ANIMATION
        if (NULL != m_pkGaugeAnimation)
        {
            m_pkGaugeAnimation->StopAnimation();
            delete m_pkGaugeAnimation;
        }
        m_pkGaugeAnimation = new GaugeAnimation(m_pkMainGauge, 300);
        m_pkGaugeAnimation->StartAnimation();
#else
        if (NULL != m_pkWheelControl)
        {
            m_pkWheelControl->ScrollToAnimated(eC_String("300"));
            m_pkWheelControl->InvalidateArea();
        }
#endif
    }
    else if (kAPI == "MiddleWheel")
    {
#if USE_SMOOTH_ANIMATION
        if (NULL != m_pkGaugeAnimation)
        {
            m_pkGaugeAnimation->StopAnimation();
            delete m_pkGaugeAnimation;
        }
        m_pkGaugeAnimation = new GaugeAnimation(m_pkMainGauge, 150);
        m_pkGaugeAnimation->StartAnimation();
#else
        if (NULL != m_pkWheelControl)
        {
            m_pkWheelControl->ScrollToAnimated(eC_String("150"));
            m_pkWheelControl->InvalidateArea();
        }
#endif
    }
    else if (kAPI == "LowWheel")
    {
#if USE_SMOOTH_ANIMATION
        if (NULL != m_pkGaugeAnimation)
        {
            m_pkGaugeAnimation->StopAnimation();
            delete m_pkGaugeAnimation;
        }
        m_pkGaugeAnimation = new GaugeAnimation(m_pkMainGauge, 0);
        m_pkGaugeAnimation->StartAnimation();
#else
        if (NULL != m_pkWheelControl)
        {
            m_pkWheelControl->ScrollToAnimated(eC_String("0"));
            m_pkWheelControl->InvalidateArea();
        }
#endif
    }
}
