#include "DemoLayer.h"

#include "GUI.h"
#include "GUIObserver.h"
#include "GUIImage.h"
#include "GUITimer.h"
#include "GUITextField.h"

#include "GfxEnv.h"

/*Always include last*/
#include "GUIMemLeakWatcher.h"

DemoLayer::DemoLayer() :
    m_pkAlphaSlider(NULL)
{
    for (eC_UInt index = 0; index < MAX_LAYER_ID; ++index)
    {
        m_kLayerData[index].Active = false;
        m_kLayerData[index].ID = 0;
        m_kLayerData[index].Running = false;
        m_kLayerData[index].Width = 0;
        m_kLayerData[index].Height = 0;
        m_kLayerData[index].BPP = 0;
    }
}

DemoLayer::~DemoLayer()
{
    DeInit();
}

void DemoLayer::Init()
{
    m_pkDialog = auto_cast<CGUICompositeObject*>(GETGUI.GetObjectByID(DLG_LAYER));
    if (NULL != m_pkDialog)
    {
        m_pkAlphaSlider = auto_cast<CGUIBaseSlider*>(m_pkDialog->GetObjectByID(SLIDER_HORIZONTAL));
        if (NULL != m_pkAlphaSlider)
            m_pkAlphaSlider->AddValueObserver(this);

        CGUICompositeObject* pkLayerContainer = auto_cast<CGUICompositeObject*>(m_pkDialog->GetObjectByID(OBJ_PAGECONTAINER));
        CGUITextField* pkNotSupported = auto_cast<CGUITextField*>(m_pkDialog->GetObjectByID(RICH_TEXT_FIELD));

        eC_Bool bHasLayers = GETGFXENV.GetNumberOfLayers() > 1;
        if (NULL != pkLayerContainer)
            pkLayerContainer->SetInvisible(!bHasLayers);

        if (NULL != pkNotSupported)
            pkNotSupported->SetInvisible(bHasLayers);

        for (eC_UInt uiLayerId = 0; uiLayerId < MAX_LAYER_ID; ++uiLayerId)
        {
            LayerConfiguration_t kLayerConfig;
            if (
                (GETGFXENV.GetLayerConfigForID(uiLayerId, kLayerConfig)) &&
                (!kLayerConfig.m_bMainLayer)
                )
            {
                m_kLayerData[uiLayerId].Active = true;
                m_kLayerData[uiLayerId].ID = kLayerConfig.m_uiLayerID;
                GETGFXENV.GetLayerBufferSize(
                    kLayerConfig.m_uiLayerID,
                    m_kLayerData[uiLayerId].Width,
                    m_kLayerData[uiLayerId].Height,
                    m_kLayerData[uiLayerId].BPP);
                m_kLayerData[uiLayerId].Running = false;
            }
            else
            {
                m_kLayerData[uiLayerId].Active = false;
            }
        }

        GETTIMER.AddAnimationCallback(100, this);
    }
}

void DemoLayer::DeInit()
{
    if (NULL != m_pkAlphaSlider)
        m_pkAlphaSlider->RemoveValueObserver(this);
    m_pkAlphaSlider = NULL;

    GETTIMER.RemoveAnimationCallback(this);
}

void DemoLayer::OnNotification(const CGUIValue& kObservedValue, const CGUIObject* const pkUpdatedObject, const eC_UInt uiX, const eC_UInt uiY)
{
    if (NULL != pkUpdatedObject)
    {
        if (pkUpdatedObject->GetID() == SLIDER_HORIZONTAL)
        {
            for (eC_UInt uiLayerIndex = 0; uiLayerIndex < MAX_LAYER_ID; ++uiLayerIndex)
            {
                if (!m_kLayerData[uiLayerIndex].Active)
                    continue;

                eC_UInt uiLayerID = m_kLayerData[uiLayerIndex].ID;
                GETGFXENV.SetLayerAlpha(uiLayerID, (eC_UByte)m_pkAlphaSlider->GetRange().GetValue());
                GETGFXENV.InvalidateLayer(uiLayerID);
            }
        }
    }
}

void DemoLayer::DoAnimate(const eC_Value& vTimes)
{
    static eC_UByte ubValue = 0;
    for (eC_UInt uiLayerIndex = 0; uiLayerIndex < MAX_LAYER_ID; ++uiLayerIndex)
    {
        eC_UInt uiLayerID = m_kLayerData[uiLayerIndex].ID;
        if (m_kLayerData[uiLayerIndex].Running)
        {
            // might change if more than 1 buffer
            void* pkLayer = GETGFXENV.GetRenderBuffer(uiLayerID);
            if (NULL != pkLayer)
            {
                if (m_kLayerData[uiLayerIndex].BPP == 2)
                {
                    eC_UShort* pusData = (eC_UShort*)pkLayer;
                    for (eC_UInt y = 0; y < m_kLayerData[uiLayerIndex].Height; ++y)
                    {
                        for (eC_UInt x = 0; x < m_kLayerData[uiLayerIndex].Width; ++x)
                        {
                            *pusData++ = (ubValue << 8) + ubValue;
                        }
                    }
                }
                else if (m_kLayerData[uiLayerIndex].BPP == 4)
                {
                    eC_UInt* puiData = (eC_UInt*)pkLayer;
                    for (eC_UInt y = 0; y < m_kLayerData[uiLayerIndex].Height; ++y)
                    {
                        for (eC_UInt x = 0; x < m_kLayerData[uiLayerIndex].Width; ++x)
                        {
                            *puiData++ = 0xff000000 + ((ubValue + 5) << 16) + ((ubValue + 10) << 8) + ubValue;
                        }
                    }
                }
            }

            ubValue += 5;
            GETGFXENV.InvalidateLayer(uiLayerID);
        }
    }
}

void DemoLayer::HandleCallAPI(const eC_String& kAPI, const eC_String kParam)
{
    if (kAPI == "LayerContent")
    {
        eC_Int iParamIndex;
        if (kParam.ToInt(iParamIndex))
        {
            if ((iParamIndex > 0) && (iParamIndex < MAX_LAYER_ID))
            {
                if (m_kLayerData[iParamIndex].Active)
                    m_kLayerData[iParamIndex].Running = !m_kLayerData[iParamIndex].Running;
            }
        }
    }
}
