# PLUGIN_NAME is different every invocation
# CAUTION: variable PLUGIN_NAME is used instead of PROJECT_NAME for include and library directories!!!

SET(GUILIANI_PLUGIN_CPP "${PATH_GUILIANI}/Share/Plugin/PluginMain.cpp")
SET(GUILIANI_PLUGIN_MEMORY_CPP "${PATH_GUILIANI}/Share/Plugin/new_delete.cpp")

ADD_LIBRARY(${PLUGIN_NAME} SHARED
  ${GUILIANI_PLUGIN_CPP}
  ${GUILIANI_GUICONFIG_CPP}
  ${GUILIANI_PLUGIN_MEMORY_CPP}
  ${GUILIANI_CUSTOM_EXTENSION_CPP}
  ${GUILIANI_CUSTOM_EXTENSION_FACTORY_CPP}
  ${COMMON_CUSTOM_EXTENSION_FUNC_CPP}
  ${CURRENT_CUSTOM_EXTENSION_CPP}
)

# plugin specific include directories
TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
  PRIVATE "${PATH_GUILIANI}/Include/CustomExtension"
  PRIVATE "${PROJECT_SOURCE_DIR}/Include/GUIConfig"
  PRIVATE "${PROJECT_SOURCE_DIR}/Common/Include/CustomExtension"
  PRIVATE "${PROJECT_SOURCE_DIR}/Include/CustomExtension"
  PRIVATE "${PATH_GUILIANI}/Share/CustomExtension"
  PRIVATE "${PATH_GUILIANI}/Share/Plugin"
)

# global include directories
# common folders
TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
  PRIVATE "${PATH_GUILIANI}/Include/Core"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Behaviour"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Command"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Control"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Exception"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Layouter"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Transition"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Utils"
  PRIVATE "${PATH_GUILIANI}/Include/Core/Video"
  PRIVATE "${PATH_GUILIANI}/Include/CustomExtension"
  PRIVATE "${PATH_GUILIANI}/Include/GUIConfig"
  PRIVATE "${PATH_GUILIANI}/Include/ImageDecoder"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/general"
)

IF(GUILIANI_PACKAGE_ADD_ANIMATION)
  TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Core/Animation"
  )
ENDIF()

IF(GUILIANI_PACKAGE_ADD_CONNECTION)
  TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Connection"
  )
ENDIF()

IF(GUILIANI_PACKAGE_ADD_STREAMING)
  TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Core/Streaming"
  )
ENDIF()

IF(STREAMRUNTIME_SDK_PROJECT)
  TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_EGAC}/Include/eC"
    PRIVATE "${PATH_EGAC}/Include/eDBG"
    PRIVATE "${PATH_EGAC}/Include/eFnt"
    PRIVATE "${PATH_EGAC}/Include/eImg"
    PRIVATE "${PATH_EGAC}/Include/eGML"
    PRIVATE "${PATH_EGAC}/Include/eMem"
  )
ELSE()
  TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_EGAC}/comps/eC/code"
    PRIVATE "${PATH_EGAC}/comps/eDBG/code"
    PRIVATE "${PATH_EGAC}/comps/eFnt/code"
    PRIVATE "${PATH_EGAC}/comps/eImg/code"
    PRIVATE "${PATH_EGAC}/comps/eGML/code"
    PRIVATE "${PATH_EGAC}/comps/eMem/code"
  )
ENDIF()

TARGET_INCLUDE_DIRECTORIES(${PLUGIN_NAME}
  PRIVATE "${PATH_GUILIANI}/Share/Plugin"
)

TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
  PRIVATE "${PATH_LIBS}/freetype/lib"
  PRIVATE "${PATH_LIBS}/webp/lib"
)

IF(NOT EGAC_BUILD_IMAGELIBS)
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_LIBS}/png/lib"
  )
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_LIBS}/jpeg/lib"
  )
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_LIBS}/zlib/lib"
  )
ENDIF()

IF(GUILIANI_ADD_DAVE_PLATFORM)
  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
      PRIVATE "${PATH_LIBS}/DAVE/win32"
      PRIVATE "${PATH_LIBS}/DAVE/Linux"
    )
  ELSE()
    TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
      PRIVATE "${PATH_LIBS}/DAVE/win64"
      PRIVATE "${PATH_LIBS}/DAVE/Linux64"
    )
  ENDIF()
ENDIF()

IF(GUILIANI_ADD_DAVEHD_PLATFORM)
  IF("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
    TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
      PRIVATE "${PATH_LIBS}/DAVEHD/win32"
      PRIVATE "${PATH_LIBS}/DAVE/Linux"
    )
  ELSE()
    TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
      PRIVATE "${PATH_LIBS}/DAVEHD/win64"
      PRIVATE "${PATH_LIBS}/DAVE/Linux64"
    )
  ENDIF()
ENDIF()

IF(GUILIANI_ADD_VULKAN_PLATFORM)
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_VULKAN}/Lib"
  )
ENDIF()

# visual studio automatically adds the build-type
IF(MSVC)
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_EGAC}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}"
    PRIVATE "${PATH_GUILIANI}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}"
  )
ELSE()
  TARGET_LINK_DIRECTORIES(${PLUGIN_NAME}
    PRIVATE "${PATH_EGAC}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}"
    PRIVATE "${PATH_GUILIANI}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}"
  )
ENDIF()

INCLUDE("${PATH_COMMON}/AddLibs.cmake")

# ========== Link the executable to the needed libraries ==========
TARGET_LINK_LIBRARIES(${PLUGIN_NAME}
  ${TOOLCHAIN_LIBS}
  Guiliani
  eGaC
  ${PLATFORM_LIBS}
)

ADD_DEPENDENCIES(${PROJECT_NAME} ${PLUGIN_NAME})

SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES COMPILE_DEFINITIONS "${COMMON_DEFINES};${EGAC_DEFINES};${GUILIANI_DEFINES}")

# RUNTIME is used for .dll-files in windows (https://cmake.org/cmake/help/v3.9/manual/cmake-buildsystem.7.html#output-artifacts)
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES RUNTIME_OUTPUT_DIRECTORY_DEBUG "${PROJECT_SOURCE_DIR}/Plugins")
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES RUNTIME_OUTPUT_DIRECTORY_RELEASE "${PROJECT_SOURCE_DIR}/Plugins")
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES RUNTIME_OUTPUT_NAME "${PLUGIN_NAME}")
# LIBRARY is used for .so-files on linux
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES LIBRARY_OUTPUT_DIRECTORY_DEBUG "${PROJECT_SOURCE_DIR}/Plugins")
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES LIBRARY_OUTPUT_DIRECTORY_RELEASE "${PROJECT_SOURCE_DIR}/Plugins")
SET_TARGET_PROPERTIES(${PLUGIN_NAME} PROPERTIES LIBRARY_OUTPUT_NAME "${PLUGIN_NAME}")
